@@ -27,7 +27,7 @@
 /*
  * Defines
  */
-#define MAX_STRING      	1024
+#define MAX_STRING              1024
 #define MAX_INPUT_LEN         	512
 #define MAX_FILEPATH            256
 #define MAX_SMALL_STR           128
@@ -38,28 +38,33 @@
 #define MAX_STATUS_LEN          128
 #define MAX_FOR_LOOPS           100
 
-#define RXBUF_LENGTH		2048
-#define MCCP_RX_LENGTH		RXBUF_LENGTH
-#define CSIPARAMS		16
+#define RXBUF_LENGTH            2048
+#define IACBUF_LENGTH           128
+#define ANSIBUF_LENGTH          128
+#define MCCP_RX_LENGTH          RXBUF_LENGTH
+#define CSIPARAMS               16
 #define ADDR_STRING_SIZE        16
 #define TIME_LEN                50
 #define SB_TIMER                5
 #define CONNECT_TIMER           1
 #define TELNET_PORT             23
-#define TIMER_TIMEOUT           1000    /* timers have a 1 second resolution */
-#define TIMER_CONT              -1      /* indicates a continuous timer */
 #define CHARSET_CONV            "UTF-8"
 
-#define DEFAULT_CHARSET         "ISO-8859-1"
+#define TIMER_TIMEOUT           1000    /* timers have a 1 second resolution */
+#define TIMER_CONT             -1       /* indicates a continuous timer */
+#define TIMER_ONESHOT          -2       /* indicates a oneshot timer */
+
+#define DEFAULT_CHAR_SET        "ISO-8859-1"
 #define DEFAULT_FONT            "monospace 10"
 #define DEFAULT_FONT_INPUT      "monospace 10"
-#define DEFAULT_WIDTH           640
-#define DEFAULT_HEIGHT          480
+#define DEFAULT_WIDTH           675
+#define DEFAULT_HEIGHT          590
 #define DEFAULT_PREF_WIDTH      600
 #define DEFAULT_PREF_HEIGHT     400
 #define DEFAULT_CAPT_WIDTH      600
 #define DEFAULT_CAPT_HEIGHT     200
 #define DEFAULT_LINE_MAX        10000
+#define DEFAULT_SPACING         0
 #define DEFAULT_FG              "grey"
 #define DEFAULT_BG              "black"
 #define DEFAULT_VAR_SIGN        '%'
@@ -67,8 +72,16 @@
 #define DEFAULT_BLOCK_OPEN      '{'
 #define DEFAULT_BLOCK_CLOSE     '}'
 #define DEFAULT_CMD_CHAR        '#'
+#define DEFAULT_SPEED_PATH      '$'
 #define DIA_NEW_WIDTH           400
 #define DIA_NEW_HEIGHT          100
+#if !defined(WIN32)
+  #define X_COMPENSATION        5
+  #define Y_COMPENSATION        23
+#else
+  #define X_COMPENSATION        4
+  #define Y_COMPENSATION        23
+#endif
 #define USER_PATH               ".gMUDix/user/"
 #define LOG_PATH                ".gMUDix/"
 
@@ -112,43 +125,43 @@
 #define TRUE                    !FALSE
 #endif
 
-#define BEEP			0x07
+#define BEEP                    0x07
 #define BS                      0x08
 #define LF                      0x0A
-#define CR			0x0D
-#define ESC			0x1B
-#define CSI			0x9B
+#define CR                      0x0D
+#define ESC                     0x1B
+#define CSI                     0x9B
 
 /* incoming ANSI codes from the server */
-#define ANSI_DEFAULT		0
-#define ANSI_BOLD		1
+#define ANSI_DEFAULT            0
+#define ANSI_BOLD               1
 #define ANSI_ITALIC             3
-#define ANSI_UNDERLINE		4
-#define ANSI_BLINK		5
-#define ANSI_REVERSE		7
-#define ANSI_BOLD_OFF		22
+#define ANSI_UNDERLINE          4
+#define ANSI_BLINK              5
+#define ANSI_REVERSE            7
+#define ANSI_BOLD_OFF           22
 #define ANSI_ITALIC_OFF         23
 #define ANSI_UNDERLINE_OFF      24
 #define ANSI_BLINK_OFF          25
 #define ANSI_REVERSE_OFF        27
-#define ANSI_FG_BLACK		30
-#define ANSI_FG_RED		31
-#define ANSI_FG_GREEN		32
-#define ANSI_FG_BROWN		33
-#define ANSI_FG_BLUE		34
-#define ANSI_FG_MAGENTA		35
-#define ANSI_FG_CYAN		36
-#define ANSI_FG_WHITE		37
-#define ANSI_FG_DEFAULT		39
-#define ANSI_BG_BLACK		40
-#define ANSI_BG_RED		41
-#define ANSI_BG_GREEN		42
-#define ANSI_BG_BROWN		43
-#define ANSI_BG_BLUE		44
-#define ANSI_BG_MAGENTA		45
-#define ANSI_BG_CYAN		46
-#define ANSI_BG_WHITE		47
-#define ANSI_BG_DEFAULT		49
+#define ANSI_FG_BLACK           30
+#define ANSI_FG_RED             31
+#define ANSI_FG_GREEN           32
+#define ANSI_FG_BROWN           33
+#define ANSI_FG_BLUE            34
+#define ANSI_FG_MAGENTA         35
+#define ANSI_FG_CYAN            36
+#define ANSI_FG_WHITE           37
+#define ANSI_FG_DEFAULT         39
+#define ANSI_BG_BLACK           40
+#define ANSI_BG_RED             41
+#define ANSI_BG_GREEN           42
+#define ANSI_BG_BROWN           43
+#define ANSI_BG_BLUE            44
+#define ANSI_BG_MAGENTA         45
+#define ANSI_BG_CYAN            46
+#define ANSI_BG_WHITE           47
+#define ANSI_BG_DEFAULT         49
 
 #define ATTR_BOLD               BIT_1
 #define ATTR_UNDERLINE          BIT_2
@@ -157,9 +170,9 @@
 #define ATTR_REVERSE            BIT_5
 
 #define IAC_OK                  0
-#define IAC_BUFFER_END         -1
-#define IAC_ESCAPED            -2
-#define IAC_MCCP_START         -3
+#define IAC_INCOMPLETE          1
+#define IAC_ESCAPED             1
+#define IAC_MCCP_START          2
 
 #define IAC_ESCAPED_SIZE        2
 #define IAC_MCCP_START_SIZE     5
@@ -168,6 +181,31 @@
 #define TRG_LOGIN              -1
 #define TRG_PASSWORD           -2
 
+#if defined(WIN32)
+#define IAC                     255             /* interpret as command: */
+#define DONT                    254             /* you are not to use option */
+#define DO                      253             /* please, you use option */
+#define WONT                    252             /* I won't use option */
+#define WILL                    251             /* I will use option */
+#define SB                      250             /* interpret as subnegotiation */
+#define GA                      249             /* you may reverse the line */
+#define EL                      248             /* erase the current line */
+#define EC                      247             /* erase the current character */
+#define AYT                     246             /* are you there */
+#define AO                      245             /* abort output--but let prog finish */
+#define IP                      244             /* interrupt process--permanently */
+#define BREAK                   243             /* break */
+#define DM                      242             /* data mark--for connect. cleaning */
+#define NOP                     241             /* nop */
+#define SE                      240             /* end sub negotiation */
+#define EOR                     239             /* end of record (transparent mode) */
+#define ABORT                   238             /* Abort process */
+#define SUSP                    237             /* Suspend process */
+#define xEOF                    236             /* End of file: EOF is already used... */
+
+#define SYNCH                   242             /* for telfunc calls */
+#endif
+
 /* bitwise flags for user->flags */
 #define FLG_SAVE_FLAGS_MASK     0xFFFF
 #define FLG_CMD_OUTPUT_MUTE     BIT_1
@@ -185,6 +223,7 @@ typedef struct trigger_type     TRIGGER;
 typedef struct user_type        USER;
 typedef struct var_type         VAR;
 typedef struct capt_window_type CAPT_WINDOW;
+typedef struct history_type     HISTORY;
 
 typedef unsigned char           bool;
 
@@ -215,8 +254,7 @@ typedef enum
 } ANSI_COLORS;
 
 
-#define COLOR_NONE              NR_ALL_COLORS
-
+#define COLOR_NONE  NR_ALL_COLORS
 
 /* default colors for foreground/background (no tags required) */
 typedef enum
@@ -255,6 +293,28 @@ typedef enum
 } NET_CODE;
 
 
+/* ANSI reception processing states */
+typedef enum
+{
+    ANSI_STATE_IDLE,
+    ANSI_STATE_ESC,
+    ANSI_STATE_PARAMS
+} ANSI_RX_STATE;
+
+
+/* rx processing states */
+typedef enum
+{
+    RX_PROC_STATE_DATA,
+    RX_PROC_STATE_IAC,
+    RX_PROC_STATE_SB,
+    RX_PROC_STATE_WILL,
+    RX_PROC_STATE_WONT,
+    RX_PROC_STATE_DO,
+    RX_PROC_STATE_DONT
+} RX_PRC_STATE;
+
+
 /* customizable characters */
 typedef enum 
 {
@@ -263,6 +323,7 @@ typedef enum
     CUST_BLOCK_OPEN,
     CUST_BLOCK_CLOSE,
     CUST_CMD_CHAR,
+    CUST_SPEED_PATH,
     NR_CUSTOM_CHARS
 } CUST_CHARS;
 
@@ -287,10 +348,11 @@ extern void       cmd_input_echo            (USER *user, int index, char *args);
 /* history.c */
 extern void       add_to_history            (USER *user, gchar *input);
 extern void       init_history              (USER *user);
+extern void       cleanup_history           (USER *user);
 
 /* log.c */
-extern bool       open_log		    (USER *user, char *filename);
-extern bool       close_log		    (USER *user);
+extern bool       open_log                  (USER *user, char *filename, bool date);
+extern bool       close_log                 (USER *user);
 extern void       write_log                 (USER *user, gchar *data, gsize length);
 
 /* macro.c */
@@ -344,7 +406,7 @@ extern void       cleanup_tabs_list         (USER *user);
 extern void       check_tab                 (USER *user, gchar *buffer);
 
 /* telnet.c */
-extern int        check_iac                 (USER *user, unsigned char *pIac, unsigned char *pEnd);
+extern int        check_iac                 (USER *user, unsigned char *pStart, unsigned char *pEnd, int *length);
 extern void       send_naws                 (USER *user);
 
 /* timer.c */
@@ -411,6 +473,7 @@ extern void       gui_color_set_def         (GdkColor *color, ANSI_COLORS index)
 extern void       gui_user_dialog_file      (USER *user);
 extern void       gui_main_update_row       (USER *user);
 extern void       gui_font_update           (USER *user, DEF_FONTS index);
+extern void       gui_font_spacing_update   (USER *user);
 extern void       gui_alias_add_pref        (USER *user, ALIAS *alias);
 extern void       gui_macro_add_pref        (USER *user, MACRO *macro);
 extern void       gui_path_add_pref         (USER *user, PATH *path);
@@ -419,11 +482,14 @@ extern void       gui_timer_add_pref        (USER *user, TIMER *timer);
 extern void       gui_trigger_add_pref      (USER *user, TRIGGER *trigger);
 extern void       gui_var_add_pref          (USER *user, VAR *var);
 extern void       gui_timer_update_list     (USER *user);
+extern void       gui_timer_remove          (USER *user, TIMER *search);
 extern void       gui_set_window_title      (USER *user);
 extern void       gui_user_backspace        (USER *user);
 extern gboolean   gui_connection_status     (USER *user);
 extern void       gui_add_to_capt           (USER *user, gchar *title, gchar *buf, gsize len, guint attrib);
 extern void       gui_user_get_xy           (USER *user, gint *x, gint *y);
+extern bool       gui_cls_capt              (USER *user, gchar *title);
+extern void       gui_user_cls              (USER *user);
 
 extern GtkWidget *gui_alias_create          (USER *user);
 extern GtkWidget *gui_macro_create          (USER *user);
@@ -447,23 +513,31 @@ extern USER      *user_list;
 
 /* macros */
 #define exit_mudix()                    (gtk_main_quit())
-#define get_prev_history_update(user)   (((user)->pGetHist == &(user)->history[0])? \
-                                         *((user)->pGetHist = &(user)->history[MAX_HISTORY-1]): \
-                                         *(--((user)->pGetHist)))
-#define get_next_history_update(user)   (((user)->pGetHist == &(user)->history[MAX_HISTORY-1])? \
-                                         *((user)->pGetHist = &(user)->history[0]): \
-                                         *(++((user)->pGetHist)))
+#define get_prev_history_update(user)   (((user)->pGetHist = (user)->pGetHist->prev)->str)
+#define get_next_history_update(user)   (((user)->pGetHist = (user)->pGetHist->next)->str)
 #define get_sb_view(user)               (gtk_bin_get_child(GTK_BIN((user)->gui_user.g_scrollback)))
+#define destroy_with_signal(object)     (g_signal_emit_by_name(G_OBJECT(object), "destroy"))
 #define VALID_ANSI(color)               ((color) != ANSI_COLOR_NONE)
 #define MCCP_USER(user)                 ((user)->net.stream)        
 
 
+struct history_type
+{
+    HISTORY    *next;
+    HISTORY    *prev;
+    gchar      *str;
+};
+
+
 /* data structure for ansi coding */
 typedef struct
 {
     guchar         fg;
     guchar         bg;
     guint          attrib;
+    char           parambuf[ANSIBUF_LENGTH];
+    char          *paramp;
+    ANSI_RX_STATE  state;
 } ANSI_T;
 
 
@@ -475,6 +549,7 @@ typedef struct
     GtkWidget     *g_scrollback;
     GtkWidget     *g_status;
     GtkWidget     *g_input;
+    GtkWidget     *g_combo;
     GtkTextBuffer *g_buffer;
     GtkTextMark   *g_mark;
     GtkTextTag    *g_fg_color_tags[NR_ALL_COLORS];
@@ -482,6 +557,7 @@ typedef struct
     GtkTextTag    *g_underline_tag;
     GtkTextTag    *g_italic_tag;
     GtkTextIter    input_iter;
+    GtkWrapMode    wrap_mode;
     GdkColor       default_color[NR_DEF_COLORS];
     GdkColor       colors[NR_ALL_COLORS];
     gchar         *fonts[NR_FONTS];
@@ -491,8 +567,9 @@ typedef struct
     gint           win_pos_y;
     gint           input_select_start;
     gint           input_select_end;
-    guint          lines_max;
-    ANSI_T         ansi;
+    gint           lines_max;
+    gint           space_above;
+    gint           space_below;
 } GUI_USER;
 
 
@@ -507,11 +584,22 @@ typedef struct
 } GUI_PREF;
 
 
+/* data structure for the trigger GUI */
+typedef struct
+{
+    GtkWidget     *entry1;
+    GtkWidget     *entry2;
+    GtkWidget     *spin;
+} GUI_TRIGGER;
+
+
 /* network related data structure */
 typedef struct
 {
     unsigned char *rxbuf;
     unsigned char *rxp;
+    unsigned char *iacbuf;
+    unsigned char *iacp;
     z_stream      *stream;
     unsigned char *mccp_out;
     unsigned char *mccp_outp;
@@ -528,6 +616,7 @@ typedef struct
     guint          recon_id;
     guint          port;
     gint           sock;
+    RX_PRC_STATE   rx_proc_state;
     NET_CODE       status;
 } NETWORK;
 
@@ -538,6 +627,7 @@ struct user_type
     USER          *next;
     GUI_USER       gui_user;
     GUI_PREF       gui_pref;
+    GUI_TRIGGER    gui_trigger;
     NETWORK        net;
     ALIAS         *alias_list;
     MACRO         *macro_list;
@@ -547,9 +637,8 @@ struct user_type
     TRIGGER       *trigger_list;
     VAR           *vars_list;
     FILE          *logfile;
-    gchar         *history[MAX_HISTORY];
-    gchar        **pCurHist;	
-    gchar        **pGetHist;
+    HISTORY       *pCurHist;
+    HISTORY       *pGetHist;
     gchar         *filename;
     gchar         *session;
     gchar         *trigger_buf;
@@ -558,6 +647,7 @@ struct user_type
     gunichar       custom_chars[NR_CUSTOM_CHARS];
     guint          id;
     guint          flags;
+    ANSI_T         ansi;
 };
 
 
@@ -567,6 +657,7 @@ struct trigger_type
     TRIGGER     *next;
     gchar       *input;
     gchar       *response;
+    GtkTextTag  *color_tag;
     gchar 	*pArg;
     gchar        arg[MAX_STRING];
     gint         level;
