@@ -19,7 +19,9 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <unistd.h>
+#if !defined(WIN32)
+  #include <unistd.h>
+#endif
 #include <ctype.h>
 #include "mudix.h"
 #include "file.h"
@@ -32,6 +34,7 @@ static char gl_arg2[MAX_STRING];
 
 static void cmd_alias(USER *user, int index, char *args);
 static void cmd_capture(USER *user, int index, char *args);
+static void cmd_cls(USER *user, int index, char *args);
 static void cmd_command(USER *user, int index, char *args);
 static void cmd_cont_timer(USER *user, int index, char *args);
 static void cmd_distrig(USER *user, int index, char *args);
@@ -42,9 +45,12 @@ static void cmd_help(USER *user, int index, char *args);
 static void cmd_if(USER *user, int index, char *args);
 static void cmd_log(USER *user, int index, char *args);
 static void cmd_macro(USER *user, int index, char *args);
+static void cmd_oneshot(USER *user, int index, char *args);
 static void cmd_path(USER *user, int index, char *args);
 static void cmd_read(USER *user, int index, char *args);
-static void cmd_shell(USER *user, int index, char *args);
+#if !defined(WIN32)
+  static void cmd_shell(USER *user, int index, char *args);
+#endif
 static void cmd_stop_timer(USER *user, int index, char *args);
 static void cmd_tab(USER *user, int index, char *args);
 static void cmd_timer(USER *user, int index, char *args);
@@ -58,8 +64,10 @@ const struct cmd_table_type cmd_table[] =
             "Create or update an alias.\n" },
     { "capture", cmd_capture, "{title} <color> {text (optional)}",
             "Capture text or current line into a capture window.\n" },
+    { "cls", cmd_cls, "{title of capture window (optional)}",
+            "Clear the user window or clear the capture window identified by its title.\n" },
     { "cmd", cmd_command, "<id> {command line}",
-            "Process the command line with another user that has 'id'.\n" },
+            "Process the command line on the user window identified by 'id'.\n" },
     { "cont_timer", cmd_cont_timer, "<time>",
             "Continue all timers with the given time.\n" },
     { "distrig", cmd_distrig, "<level>",
@@ -76,18 +84,22 @@ const struct cmd_table_type cmd_table[] =
             "Defines whether the input bar is cleared after enter. No argument toggles this flag.\n" },
     { "input_echo", cmd_input_echo, "{on/off}",
             "Defines whether the input is echoed to the main window. No argument toggles this flag.\n" },
-    { "log", cmd_log, "{filename (optional)}",
-            "Starts logging to a file or stops logging (no filename).\n" },
+    { "log", cmd_log, "{filename} {date (optional)}",
+            "Starts logging to a file or stops logging (to stop keep filename empty).\n" },
     { "macro", cmd_macro, "{key string} {response}",
             "Create or update a macro.\n" },
     { "mute", cmd_mute, "{on/off}",
             "Mutes/De-mutes the command ouput. No argument toggles this flag.\n" },
+    { "oneshot", cmd_oneshot, "<time> {response}",
+            "Create a timer (in seconds) which is deleted after it expires.\n" },
     { "path", cmd_path, "{path name} {path string}",
             "Create or execute (only the path name) a path.\n" },
     { "read", cmd_read, "{filename}",
             "Read (and execute) a file containing MUDix-commands.\n" },
+#if !defined(WIN32)
     { "shell", cmd_shell, "<command>",
             "Execute command in a background shell.\n" },
+#endif
     { "stop_timer", cmd_stop_timer, "<time>",
             "Pause all timers with the given time.\n" },
     { "tab", cmd_tab, "{tab name}",
@@ -117,11 +129,11 @@ static int cmd_lookup(char *cmd)
 
     while (cmd_table[i].cmd) 
     {
-	if (!strncmp(cmd, cmd_table[i].cmd, strlen(cmd)))
+        if (!strncmp(cmd, cmd_table[i].cmd, strlen(cmd)))
         {
-	    return i;
+            return i;
         }
-	i++;
+        i++;
     }
 
     return -1;
@@ -137,7 +149,7 @@ bool do_command(USER *user, char *cmdline)
 
     if (index < 0)
     {
-	return FALSE;
+        return FALSE;
     }
 
     (*(cmd_table[index].function))(user, index, cmdline); 
@@ -235,9 +247,9 @@ static void cmd_distrig(USER *user, int index, char *arglist)
     level = atoi(gl_arg1);
     for (trig = user->trigger_list; trig; trig = trig->next) 
     {
-	if (trig->level == level)
+        if (trig->level == level)
         {
-	    trig->enabled = FALSE;
+            trig->enabled = FALSE;
             count++;
         }
     }
@@ -262,9 +274,9 @@ static void cmd_enatrig(USER *user, int index, char *arglist)
     level = atoi(gl_arg1);
     for (trig = user->trigger_list; trig; trig = trig->next) 
     {
-	if (trig->level == level)
+        if (trig->level == level)
         {
-	    trig->enabled = TRUE;
+            trig->enabled = TRUE;
             count++;
         }
     }
@@ -298,22 +310,24 @@ static void cmd_log(USER *user, int index, char *arglist)
     get_arg(user, arglist, gl_arg1);
     if (gl_arg1[0]) 
     {
-	if (!open_log(user, gl_arg1))
+        get_arg(user, arglist, gl_arg2);
+
+        if (!open_log(user, gl_arg1, gl_arg2[0]? TRUE: FALSE))
         {
             syntax_msg(user, index);
         }
-	else if (!(user->flags & FLG_CMD_OUTPUT_MUTE))
+        else if (!(user->flags & FLG_CMD_OUTPUT_MUTE))
         {
-	    gui_info_message(user, "Log started.\n");
+            gui_info_message(user, "Log started.\n");
         }
     }
     else 
     {
-	if (!close_log(user))
+        if (!close_log(user))
         {
             syntax_msg(user, index);
         }
-	else if (!(user->flags & FLG_CMD_OUTPUT_MUTE))
+        else if (!(user->flags & FLG_CMD_OUTPUT_MUTE))
         {
             gui_info_message(user, "Log closed.\n");
         }
@@ -369,9 +383,9 @@ static void cmd_path(USER *user, int index, char *arglist)
     {
         for (path = user->path_list; path; path = path->next) 
         {
-	    if (!strcmp(gl_arg1, path->name))
+            if (!strcmp(gl_arg1, path->name))
             {
-	        break;
+                break;
             }
         }
 
@@ -411,13 +425,13 @@ static void cmd_read(USER *user, int index, char *arglist)
     if (!gl_arg1[0]) 
     {
         syntax_msg(user, index);
-	return;
+        return;
     }
 
     if (!(fp = fopen(gl_arg1, "r"))) 
     {
-	gui_info_message(user, "Failed to open file.\n");
-	return;
+        gui_info_message(user, "Failed to open file.\n");
+        return;
     }
 
     if (!(user->flags & FLG_CMD_OUTPUT_MUTE))
@@ -428,11 +442,11 @@ static void cmd_read(USER *user, int index, char *arglist)
 
     while ((line = fread_to_eol(fp))) 
     {
-	if (*line == '\0')
+        if (*line == '\0')
         {
-	    break;
+            break;
         }
-	process_input(user, line, NULL);	
+        process_input(user, line, NULL);        
     }
 
     fclose(fp);
@@ -480,7 +494,7 @@ static void cmd_timer(USER *user, int index, char *arglist)
     reload  = atoi(gl_arg1);
 
     get_arg(user, arglist, gl_arg1);
-    if (gl_arg1[0] == '\0' || timer < 0)
+    if (gl_arg1[0] == '\0' || time < 0)
     {
         syntax_msg(user, index);
         return;
@@ -502,6 +516,37 @@ static void cmd_timer(USER *user, int index, char *arglist)
 }
 
 
+static void cmd_oneshot(USER *user, int index, char *arglist)
+{
+    TIMER *timer;
+    int    time;
+
+    arglist = get_arg(user, arglist, gl_arg1);
+    time    = atoi(gl_arg1);
+    
+    get_arg(user, arglist, gl_arg1);
+    if (gl_arg1[0] == '\0' || time < 0)
+    {
+        syntax_msg(user, index);
+        return;
+    }
+
+    if (!(timer = create_timer(user, gl_arg1, time, TIMER_ONESHOT)))
+    {
+        exit_mudix();
+    }
+    
+    if (!(user->flags & FLG_CMD_OUTPUT_MUTE))
+    {
+        sprintf(gl_arg1, "Oneshot timer created for %ds.\n", time);
+        gui_info_message(user, gl_arg1);
+    }
+
+    gui_timer_add_pref(user, timer);
+    return;
+}
+
+
 static void cmd_stop_timer(USER *user, int index, char *arglist)
 {
     TIMER *timer;
@@ -872,6 +917,7 @@ void cmd_input_clear(USER *user, int index, char *args)
 }
 
 
+#if !defined(WIN32)
 void cmd_shell(USER *user, int index, char *args)
 {   
     pid_t pid = fork();
@@ -899,3 +945,22 @@ void cmd_shell(USER *user, int index, char *args)
         /* this is the parent */
     }
 }
+#endif
+
+
+void cmd_cls(USER *user, int index, char *args)
+{   
+    get_arg(user, args, gl_arg1);
+
+    if (gl_arg1[0])
+    {
+        if (gui_cls_capt(user, gl_arg1) && !(user->flags & FLG_CMD_OUTPUT_MUTE))
+        {
+            gui_info_message(user, "Capture window cleared.\n");
+        }
+    }
+    else
+    {
+        gui_user_cls(user);
+    }
+}
