@@ -22,8 +22,12 @@
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <dirent.h>
-#include <unistd.h>
+#if !defined(WIN32)
+  #include <dirent.h>
+  #include <unistd.h>
+#else
+  #include <windows.h>
+#endif
 #include <time.h>
 #include "mudix.h"
 
@@ -33,11 +37,11 @@ static GtkWidget       *gui_main_window;
 static GtkTreeModel    *gui_main_model;
 
 
-#define BUTTON_CONNECT  "Connect"
-#define BUTTON_EDIT     "Edit"
-#define BUTTON_NEW      "New"
-#define BUTTON_DELETE   "Delete"
-#define BUTTON_EXIT     "Exit"
+#define BUTTON_CONNECT  "_Connect"
+#define BUTTON_EDIT     "_Edit"
+#define BUTTON_NEW      "_New"
+#define BUTTON_DELETE   "_Delete"
+#define BUTTON_EXIT     "E_xit"
 
 
 typedef enum
@@ -71,6 +75,9 @@ static void gui_main_connect(char *path)
 
         /* make the connection to the server */
         gui_user_connect(user);
+
+        /* close the user selector */
+        destroy_with_signal(gui_main_window);
     }
     else
     {
@@ -148,16 +155,17 @@ static void gui_main_dialog_new(void)
     GtkWidget *label;
     GtkWidget *sep;
     gint       response;
+    bool       fDestroyMain = FALSE;
 
     /* create a dialog popup window */
     dialog = gtk_dialog_new_with_buttons("New User",
-    					 GTK_WINDOW(gui_main_window),
-					 GTK_DIALOG_MODAL| GTK_DIALOG_DESTROY_WITH_PARENT,
-					 GTK_STOCK_OK,
-					 GTK_RESPONSE_OK,
+                                         GTK_WINDOW(gui_main_window),
+                                         GTK_DIALOG_MODAL| GTK_DIALOG_DESTROY_WITH_PARENT,
+                                         GTK_STOCK_OK,
+                                         GTK_RESPONSE_OK,
                                          GTK_STOCK_CANCEL,
                                          GTK_RESPONSE_CANCEL,
-					 NULL);
+                                         NULL);
 
     label = gtk_label_new("Session name:");
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, FALSE, FALSE, 0);
@@ -281,7 +289,11 @@ static void gui_main_dialog_new(void)
             {
                 struct stat buf;
 
+#if !defined(WIN32)
                 sprintf(path, "%s/" USER_PATH "%s_%d.usr", getenv("HOME"), str, i);
+#else
+                sprintf(path, "./%s_%d.usr", str, i);
+#endif
                 if (stat(path, &buf))
                 {
                     /* file does not exist or there was an error */
@@ -333,10 +345,18 @@ static void gui_main_dialog_new(void)
 
             /* initial save to create the filename :) */
             save_user(user);
+
+            fDestroyMain = TRUE;
         }
     }
 
     gtk_widget_destroy(dialog);
+
+    if (fDestroyMain)
+    {
+        /* close the user selector (IMPORTANT: after gtk_widget_destroy(dialog)) */
+        destroy_with_signal(gui_main_window);
+    }
 }
 
 
@@ -474,6 +494,61 @@ static gboolean gui_main_row_do_update(GtkTreeModel *model,
 }
 
 
+#if defined(WIN32)
+/* scandir function to be compatible with the linux scandir function */
+
+struct dirent
+{
+  char d_name[MAX_INPUT_LEN];
+};
+
+static int scandir(char *path, struct dirent ***namelist)
+{
+    HANDLE          Hnd;
+    WIN32_FIND_DATA WFD;
+    int             i = 0;
+
+    /* max 128 pointers to dirent structs */
+    *namelist = malloc(128*sizeof(void *));
+
+    /* set the new current directory */
+    SetCurrentDirectory(path);
+
+    /* start the search */
+    Hnd = FindFirstFile("*.usr", &WFD);
+
+    if (Hnd != INVALID_HANDLE_VALUE)
+    {
+      /* browse through the directory */
+      do
+      {
+        /* check that it's not a directory */
+        if (!(WFD.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
+        {
+            struct dirent *pFile = (struct dirent *)malloc(sizeof(struct dirent));
+
+            strcpy(pFile->d_name, WFD.cFileName);
+
+            (void *)(*namelist)[i++] = pFile;
+        }
+      } while (FindNextFile(Hnd, &WFD) && i < 128);
+
+      /* end the search to this call */
+      FindClose(Hnd);
+    }
+
+    if (i == 0)
+    {
+        i--;
+        free(*namelist);
+    }
+
+    return i;
+}
+
+#endif
+
+
 static GtkTreeModel *gui_main_create_model(int *latest)
 {
     GtkListStore    *store;
@@ -485,22 +560,33 @@ static GtkTreeModel *gui_main_create_model(int *latest)
 
     /* create list store */
     store = gtk_list_store_new(NUM_COLUMNS,
-			       G_TYPE_STRING,
-			       G_TYPE_STRING,
-			       G_TYPE_INT,
+                               G_TYPE_STRING,
+                               G_TYPE_STRING,
+                               G_TYPE_INT,
                                G_TYPE_STRING,
                                G_TYPE_STRING);
 
+#if !defined(WIN32)
     /* scan our directory for files */
     sprintf(path, "%s/" USER_PATH, getenv("HOME"));
+
     n = scandir(path, &namelist, 0, alphasort);
+#else
+    /* scan our directory for files */
+    sprintf(path, ".");
+
+    n = scandir(path, &namelist);
+#endif
+
     if (n < 0)
     {
+#if !defined(WIN32)
         char dir[MAX_FILEPATH];
 
         /* our user dir is non existing - create it */
         sprintf(dir, "mkdir -p %s", path);
         system(dir);
+#endif
 
         /* set latest to -1 */
         *latest = -1;
@@ -513,12 +599,20 @@ static GtkTreeModel *gui_main_create_model(int *latest)
         {
             struct stat buf;
 
+#if !defined(WIN32)
             sprintf(path, "%s/" USER_PATH "%s", getenv("HOME"), namelist[i]->d_name);
+#else
+            sprintf(path, "./%s", namelist[i]->d_name);
+#endif
             if (stat(path, &buf) == -1)
             {
                 perror("stat");
             }
+#if !defined(WIN32)
             else if (!S_ISDIR(buf.st_mode))  /* regular file */
+#else
+            else  /* we already checked for dir */
+#endif
             {
                 char  time[TIME_LEN];
                 char *site = NULL;
@@ -553,12 +647,12 @@ static GtkTreeModel *gui_main_create_model(int *latest)
 
                 gtk_list_store_append(store, &iter);
                 gtk_list_store_set(store, &iter,
-                               COLUMN_NAME, name,
-                               COLUMN_SITE, site,
-                               COLUMN_PORT, port,
-                               COLUMN_LAST, time,
-                               COLUMN_PATH, path,
-                               -1);
+                                   COLUMN_NAME, name,
+                                   COLUMN_SITE, site,
+                                   COLUMN_PORT, port,
+                                   COLUMN_LAST, time,
+                                   COLUMN_PATH, path,
+                                   -1);
 
                 /* free allocated memory */
                 if (name)
@@ -588,38 +682,38 @@ static void gui_main_add_columns(GtkTreeView *treeview)
     renderer = gtk_cell_renderer_text_new();
     column = gtk_tree_view_column_new_with_attributes("Name",
                                                       renderer,
-						      "text",
-						      COLUMN_NAME,
-						      NULL);
+                                                      "text",
+                                                      COLUMN_NAME,
+                                                      NULL);
     gtk_tree_view_column_set_sort_column_id(column, COLUMN_NAME);
     gtk_tree_view_append_column(treeview, column);
 
     /* column for site description */
     renderer = gtk_cell_renderer_text_new();
     column = gtk_tree_view_column_new_with_attributes("Site",
-						      renderer,
-						      "text",
-						      COLUMN_SITE,
-						      NULL);
+                                                      renderer,
+                                                      "text",
+                                                      COLUMN_SITE,
+                                                      NULL);
     gtk_tree_view_column_set_sort_column_id(column, COLUMN_SITE);
     gtk_tree_view_append_column(treeview, column);
 
     /* column for port */
     renderer = gtk_cell_renderer_text_new();
     column = gtk_tree_view_column_new_with_attributes("Port",
-						      renderer,
-						      "text",
-						      COLUMN_PORT,
-						      NULL);
+                                                      renderer,
+                                                      "text",
+                                                      COLUMN_PORT,
+                                                      NULL);
     gtk_tree_view_append_column(treeview, column);
 
     /* column for last */
     renderer = gtk_cell_renderer_text_new();
     column = gtk_tree_view_column_new_with_attributes("Last Connect",
-						      renderer,
-						      "text",
-						      COLUMN_LAST,
-						      NULL);
+                                                      renderer,
+                                                      "text",
+                                                      COLUMN_LAST,
+                                                      NULL);
     gtk_tree_view_column_set_sort_column_id(column, COLUMN_LAST);
     gtk_tree_view_append_column(treeview, column);
 }
@@ -690,9 +784,10 @@ void gui_setup_main_window(void)
     gtk_window_set_title(GTK_WINDOW(gui_main_window), 
                          "gMUDix v" VERSION " by M. Boomstra");
     g_signal_connect(G_OBJECT(gui_main_window), "destroy",
-	             G_CALLBACK(gui_destroy_main_window_callback), NULL);
+                     G_CALLBACK(gui_destroy_main_window_callback), NULL);
     gtk_container_set_border_width(GTK_CONTAINER(gui_main_window), 2);
     gtk_window_set_default_size(GTK_WINDOW(gui_main_window), 400, 165);
+    gtk_window_set_position(GTK_WINDOW(gui_main_window), GTK_WIN_POS_CENTER_ALWAYS);
 
     /* create a vertical box where the complete editor is inside */
     mainbox = gtk_vbox_new(FALSE, 2);
@@ -740,6 +835,10 @@ void gui_setup_main_window(void)
 
     /* create the buttons */
     button = gtk_button_new_with_label(BUTTON_CONNECT);
+    gtk_button_set_use_underline(GTK_BUTTON(button) , TRUE);
+    /* activate is when the mnemonic key is pressed. */
+    g_signal_connect(G_OBJECT(button), "activate",
+                     G_CALLBACK(gui_main_button_release), view);    
     g_signal_connect(G_OBJECT(button), "released",
                      G_CALLBACK(gui_main_button_release), view);    
 
@@ -748,6 +847,10 @@ void gui_setup_main_window(void)
 
     /* create the buttons */
     button = gtk_button_new_with_label(BUTTON_EDIT);
+    gtk_button_set_use_underline(GTK_BUTTON(button) , TRUE);
+    /* activate is when the mnemonic key is pressed. */
+    g_signal_connect(G_OBJECT(button), "activate",
+                     G_CALLBACK(gui_main_button_release), view);    
     g_signal_connect(G_OBJECT(button), "released",
                      G_CALLBACK(gui_main_button_release), view);    
 
@@ -756,6 +859,10 @@ void gui_setup_main_window(void)
 
     /* create the buttons */
     button = gtk_button_new_with_label(BUTTON_NEW);
+    gtk_button_set_use_underline(GTK_BUTTON(button) , TRUE);
+    /* activate is when the mnemonic key is pressed. */
+    g_signal_connect(G_OBJECT(button), "activate",
+                     G_CALLBACK(gui_main_button_release), view);    
     g_signal_connect(G_OBJECT(button), "released",
                      G_CALLBACK(gui_main_button_release), view);    
 
@@ -763,6 +870,10 @@ void gui_setup_main_window(void)
     gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 2);
 
     button = gtk_button_new_with_label(BUTTON_DELETE);
+    gtk_button_set_use_underline(GTK_BUTTON(button) , TRUE);
+    /* activate is when the mnemonic key is pressed. */
+    g_signal_connect(G_OBJECT(button), "activate",
+                     G_CALLBACK(gui_main_button_release), view);    
     g_signal_connect(G_OBJECT(button), "released",
                      G_CALLBACK(gui_main_button_release), view);    
 
@@ -770,6 +881,10 @@ void gui_setup_main_window(void)
     gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 2);
 
     button = gtk_button_new_with_label(BUTTON_EXIT);
+    gtk_button_set_use_underline(GTK_BUTTON(button) , TRUE);
+    /* activate is when the mnemonic key is pressed. */
+    g_signal_connect(G_OBJECT(button), "activate",
+                     G_CALLBACK(gui_main_button_release), view);    
     g_signal_connect(G_OBJECT(button), "released",
                      G_CALLBACK(gui_destroy_main_window_callback), NULL);    
 
