@@ -24,13 +24,13 @@
 
 
 /* the possible button labels */
-#define BUTTON_NEW      "New"
-#define BUTTON_DELETE   "Delete"
-#define BUTTON_APPLY    "Apply"
-#define BUTTON_CONTINUE "Continue"
-#define BUTTON_STOP     "Stop"
-#define BUTTON_SYNC     "Sync"
-#define BUTTON_EXIT     "Exit"
+#define BUTTON_NEW      "_New"
+#define BUTTON_DELETE   "_Delete"
+#define BUTTON_APPLY    "_Apply"
+#define BUTTON_CONTINUE "_Continue"
+#define BUTTON_STOP     "_Stop"
+#define BUTTON_SYNC     "S_ync"
+#define BUTTON_EXIT     "E_xit"
 
 typedef enum
 {
@@ -67,8 +67,8 @@ static void gui_timer_button_release(GtkButton *button, USER *user)
 
         /* then retrieve the data in the widgets */
         input = (gchar *)gtk_entry_get_text(GTK_ENTRY(entry));
-        val1  = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spin1));
-        val2  = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spin2));
+        val1  = (gint)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spin1));
+        val2  = (gint)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spin2));
 
         if (!input[0] || !val1)
         {
@@ -149,8 +149,8 @@ static void gui_timer_button_release(GtkButton *button, USER *user)
 
         /* then retrieve the data in the widgets */
         input = (gchar *)gtk_entry_get_text(GTK_ENTRY(entry));
-        val1  = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spin1));
-        val2  = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spin2));
+        val1  = (gint)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spin1));
+        val2  = (gint)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spin2));
 
         if (!input[0] || !val1)
         {
@@ -378,6 +378,38 @@ static gboolean gui_timers_row_update(GtkTreeModel *model,
 }
 
 
+void gui_timer_remove(USER *user, TIMER *search)
+{
+    TIMER *timer;
+
+    /* only do so if the timer window is visible */
+    if (user->gui_pref.g_edit[SEL_TIMERS])
+    {
+        GtkTreeModel *model;
+        GtkTreeIter   iter;
+
+        /* retrieve the model */
+        model = gtk_tree_view_get_model(GTK_TREE_VIEW(
+                gtk_bin_get_child(GTK_BIN(gui_get_nth_child(
+                user->gui_pref.g_edit[SEL_TIMERS], 0)))));
+
+        if (gtk_tree_model_get_iter_first(model, &iter))
+        {
+            do
+            {
+                gtk_tree_model_get(model, &iter, COLUMN_POINTER, &timer, -1);
+                if (timer == search)
+                {
+                    /* remove from the model and free it */
+                    gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
+                    break;
+                }
+            } while (gtk_tree_model_iter_next(model, &iter));
+        }
+    }
+}
+
+
 void gui_timer_update_list(USER *user)
 {
     /* call the row update routine for each row */
@@ -517,6 +549,10 @@ GtkWidget *gui_timer_create(USER *user)
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(reload), 0);
 
     button = gtk_button_new_with_label(BUTTON_CONTINUE);
+    gtk_button_set_use_underline(GTK_BUTTON(button) , TRUE);
+    /* activate is when the mnemonic key is pressed. */
+    g_signal_connect(G_OBJECT(button), "activate",
+                     G_CALLBACK(gui_timer_button_release), user);
     g_signal_connect(G_OBJECT(button), "released",
                      G_CALLBACK(gui_timer_button_release), user);    
 
@@ -524,6 +560,10 @@ GtkWidget *gui_timer_create(USER *user)
     gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 4);
 
     button = gtk_button_new_with_label(BUTTON_STOP);
+    gtk_button_set_use_underline(GTK_BUTTON(button) , TRUE);
+    /* activate is when the mnemonic key is pressed. */
+    g_signal_connect(G_OBJECT(button), "activate",
+                     G_CALLBACK(gui_timer_button_release), user);
     g_signal_connect(G_OBJECT(button), "released",
                      G_CALLBACK(gui_timer_button_release), user);    
 
@@ -531,6 +571,10 @@ GtkWidget *gui_timer_create(USER *user)
     gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 4);
 
     button = gtk_button_new_with_label(BUTTON_SYNC);
+    gtk_button_set_use_underline(GTK_BUTTON(button) , TRUE);
+    /* activate is when the mnemonic key is pressed. */
+    g_signal_connect(G_OBJECT(button), "activate",
+                     G_CALLBACK(gui_timer_button_release), user);
     g_signal_connect(G_OBJECT(button), "released",
                      G_CALLBACK(gui_timer_button_release), user);    
 
@@ -560,6 +604,10 @@ GtkWidget *gui_timer_create(USER *user)
 
     /* create the buttons */
     button = gtk_button_new_with_label(BUTTON_NEW);
+    gtk_button_set_use_underline(GTK_BUTTON(button) , TRUE);
+    /* activate is when the mnemonic key is pressed. */
+    g_signal_connect(G_OBJECT(button), "activate",
+                     G_CALLBACK(gui_timer_button_release), user);
     g_signal_connect(G_OBJECT(button), "released",
                      G_CALLBACK(gui_timer_button_release), user);    
 
@@ -567,6 +615,10 @@ GtkWidget *gui_timer_create(USER *user)
     gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 2);
 
     button = gtk_button_new_with_label(BUTTON_DELETE);
+    gtk_button_set_use_underline(GTK_BUTTON(button) , TRUE);
+    /* activate is when the mnemonic key is pressed. */
+    g_signal_connect(G_OBJECT(button), "activate",
+                     G_CALLBACK(gui_timer_button_release), user);
     g_signal_connect(G_OBJECT(button), "released",
                      G_CALLBACK(gui_timer_button_release), user);    
 
@@ -574,6 +626,10 @@ GtkWidget *gui_timer_create(USER *user)
     gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 2);
 
     button = gtk_button_new_with_label(BUTTON_APPLY);
+    gtk_button_set_use_underline(GTK_BUTTON(button) , TRUE);
+    /* activate is when the mnemonic key is pressed. */
+    g_signal_connect(G_OBJECT(button), "activate",
+                     G_CALLBACK(gui_timer_button_release), user);
     g_signal_connect(G_OBJECT(button), "released",
                      G_CALLBACK(gui_timer_button_release), user);    
 
@@ -581,6 +637,10 @@ GtkWidget *gui_timer_create(USER *user)
     gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 2);
 
     button = gtk_button_new_with_label(BUTTON_EXIT);
+    gtk_button_set_use_underline(GTK_BUTTON(button) , TRUE);
+    /* activate is when the mnemonic key is pressed. */
+    g_signal_connect(G_OBJECT(button), "activate",
+                     G_CALLBACK(gui_timer_button_release), user);
     g_signal_connect(G_OBJECT(button), "released",
                      G_CALLBACK(gui_timer_button_release), user);    
 
