@@ -24,12 +24,12 @@
 
 
 /* the possible button labels */
-#define BUTTON_NEW      "New"
-#define BUTTON_DELETE   "Delete"
-#define BUTTON_APPLY    "Apply"
-#define BUTTON_LOGIN    "Login Level"
-#define BUTTON_PASSWORD "Password Level"
-#define BUTTON_EXIT     "Exit"
+#define BUTTON_NEW      "_New"
+#define BUTTON_DELETE   "_Delete"
+#define BUTTON_APPLY    "_Apply"
+#define BUTTON_LOGIN    "_Login Level"
+#define BUTTON_PASSWORD "_Password Level"
+#define BUTTON_EXIT     "E_xit"
 
 typedef enum
 {
@@ -66,9 +66,9 @@ static void gui_trigger_button_release(GtkButton *button, USER *user)
         gdouble       level;
 
         /* first get a pointer to the widgets involved */
-        entry1 = gui_get_nth_child(gui_get_wid_frame(user, 0, SEL_TRIGGERS), 0);
-        entry2 = gui_get_nth_child(gui_get_wid_frame(user, 1, SEL_TRIGGERS), 0);
-        spin   = gui_get_nth_child(gui_get_wid_frame(user, 2, SEL_TRIGGERS), 0);
+        entry1   = user->gui_trigger.entry1;
+        entry2   = user->gui_trigger.entry2;
+        spin     = user->gui_trigger.spin;
 
         /* then retrieve the data in the widgets */
         input1 = (gchar *)gtk_entry_get_text(GTK_ENTRY(entry1));
@@ -137,12 +137,9 @@ static void gui_trigger_button_release(GtkButton *button, USER *user)
         gtk_tree_model_get(model, &iter, COLUMN_POINTER, &trigger, -1);
 
         /* first retrieve the entries */
-        entry1 = (gchar *)gtk_entry_get_text(GTK_ENTRY(
-                          gui_get_nth_child(gui_get_wid_frame(user, 0, SEL_TRIGGERS), 0)));
-        entry2 = (gchar *)gtk_entry_get_text(GTK_ENTRY(
-                          gui_get_nth_child(gui_get_wid_frame(user, 1, SEL_TRIGGERS), 0)));
-        level  = gtk_spin_button_get_value(GTK_SPIN_BUTTON(
-                         gui_get_nth_child(gui_get_wid_frame(user, 2, SEL_TRIGGERS), 0)));
+        entry1 = (gchar *)gtk_entry_get_text(GTK_ENTRY(user->gui_trigger.entry1));
+        entry2 = (gchar *)gtk_entry_get_text(GTK_ENTRY(user->gui_trigger.entry2));
+        level  = gtk_spin_button_get_value(GTK_SPIN_BUTTON(user->gui_trigger.spin));
 
         if (!entry1[0] || !entry2[0])
         {
@@ -152,7 +149,7 @@ static void gui_trigger_button_release(GtkButton *button, USER *user)
         /* update the trigger */
         set_trigger(user, trigger, entry1, entry2);
         trigger->level = (gint)level;
-        
+
         /* set the trigger view */
         gtk_list_store_set(GTK_LIST_STORE(model), &iter,
                            COLUMN_ENABLE, trigger->enabled,
@@ -165,8 +162,7 @@ static void gui_trigger_button_release(GtkButton *button, USER *user)
         GtkSpinButton *spin;
         
         /* grab the spin button and update its value */
-        spin = GTK_SPIN_BUTTON(gui_get_nth_child(
-                               gui_get_wid_frame(user, 2, SEL_TRIGGERS), 0));
+        spin = GTK_SPIN_BUTTON(user->gui_trigger.spin);
 
         gtk_spin_button_set_value(spin, TRG_LOGIN);
     }
@@ -175,8 +171,7 @@ static void gui_trigger_button_release(GtkButton *button, USER *user)
         GtkSpinButton *spin;
         
         /* grab the spin button and update its value */
-        spin = GTK_SPIN_BUTTON(gui_get_nth_child(
-                               gui_get_wid_frame(user, 2, SEL_TRIGGERS), 0));
+        spin = GTK_SPIN_BUTTON(user->gui_trigger.spin);
 
         gtk_spin_button_set_value(spin, TRG_PASSWORD);
     }
@@ -236,14 +231,9 @@ static void gui_trigger_row_selected(GtkTreeView *treeview, USER *user)
     gtk_tree_model_get(model, &iter, COLUMN_POINTER, &trigger, -1);
 
     /* set the entries and spin button */
-    gtk_entry_set_text(GTK_ENTRY(gui_get_nth_child(
-                              gui_get_wid_frame(user, 0, SEL_TRIGGERS), 0)),
-                              trigger->input);
-    gtk_entry_set_text(GTK_ENTRY(gui_get_nth_child(
-                              gui_get_wid_frame(user, 1, SEL_TRIGGERS), 0)),
-                              trigger->response);
-    gtk_spin_button_set_value(GTK_SPIN_BUTTON(gui_get_nth_child(
-                              gui_get_wid_frame(user, 2, SEL_TRIGGERS), 0)),
+    gtk_entry_set_text(GTK_ENTRY(user->gui_trigger.entry1), trigger->input);
+    gtk_entry_set_text(GTK_ENTRY(user->gui_trigger.entry2), trigger->response);
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(user->gui_trigger.spin), 
                               (gdouble)trigger->level);
 }
 
@@ -268,8 +258,8 @@ static GtkTreeModel *gui_trigger_create_model(USER *user)
            for efficiency! */
         gtk_list_store_append(store, &iter);
         gtk_list_store_set(store, &iter,
-                           COLUMN_ENABLE, trigger->enabled,
-			   COLUMN_LEVEL, trigger->level,
+                           COLUMN_ENABLE,  trigger->enabled,
+			   COLUMN_LEVEL,   trigger->level,
 			   COLUMN_TRIGGER, trigger->input,
                            COLUMN_POINTER, trigger, -1);
     }
@@ -399,7 +389,7 @@ GtkWidget *gui_trigger_create(USER *user)
     hbox = gtk_hbox_new(TRUE, 0);
 
     /* create the trigger entry */
-    trigger  = gtk_entry_new();
+    user->gui_trigger.entry1 = trigger = gtk_entry_new();
 
     /* put trigger in hbox */
     gtk_box_pack_start(GTK_BOX(hbox), trigger, TRUE, TRUE, 0);
@@ -417,7 +407,7 @@ GtkWidget *gui_trigger_create(USER *user)
     hbox = gtk_hbox_new(TRUE, 0);
 
     /* create the response entry */
-    response = gtk_entry_new();
+    user->gui_trigger.entry2 = response = gtk_entry_new();
 
     /* put response in hbox */
     gtk_box_pack_start(GTK_BOX(hbox), response, TRUE, TRUE, 0);
@@ -435,7 +425,7 @@ GtkWidget *gui_trigger_create(USER *user)
     hbox = gtk_hbox_new(FALSE, 0);
 
     /* create a spin button for the level */
-    spin = gtk_spin_button_new_with_range(-100000, 100000, 1);
+    user->gui_trigger.spin = spin = gtk_spin_button_new_with_range(-100000, 100000, 1);
 
     gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spin), TRUE);
     gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
@@ -451,12 +441,20 @@ GtkWidget *gui_trigger_create(USER *user)
 
     /* create buttons for login and password triggers */
     button = gtk_button_new_with_label(BUTTON_LOGIN);
+    gtk_button_set_use_underline(GTK_BUTTON(button) , TRUE);
+    /* activate is when the mnemonic key is pressed. */
+    g_signal_connect(G_OBJECT(button), "activate",
+                     G_CALLBACK(gui_trigger_button_release), user);    
     g_signal_connect(G_OBJECT(button), "released",
                      G_CALLBACK(gui_trigger_button_release), user);    
     /* put the button inside the horizontal box */
     gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 4);
 
     button = gtk_button_new_with_label(BUTTON_PASSWORD);
+    gtk_button_set_use_underline(GTK_BUTTON(button) , TRUE);
+    /* activate is when the mnemonic key is pressed. */
+    g_signal_connect(G_OBJECT(button), "activate",
+                     G_CALLBACK(gui_trigger_button_release), user);    
     g_signal_connect(G_OBJECT(button), "released",
                      G_CALLBACK(gui_trigger_button_release), user);    
     /* put the button inside the horizontal box */
@@ -472,6 +470,10 @@ GtkWidget *gui_trigger_create(USER *user)
 
     /* create the buttons */
     button = gtk_button_new_with_label(BUTTON_NEW);
+    gtk_button_set_use_underline(GTK_BUTTON(button) , TRUE);
+    /* activate is when the mnemonic key is pressed. */
+    g_signal_connect(G_OBJECT(button), "activate",
+                     G_CALLBACK(gui_trigger_button_release), user);    
     g_signal_connect(G_OBJECT(button), "released",
                      G_CALLBACK(gui_trigger_button_release), user);    
 
@@ -479,6 +481,10 @@ GtkWidget *gui_trigger_create(USER *user)
     gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 2);
 
     button = gtk_button_new_with_label(BUTTON_DELETE);
+    gtk_button_set_use_underline(GTK_BUTTON(button) , TRUE);
+    /* activate is when the mnemonic key is pressed. */
+    g_signal_connect(G_OBJECT(button), "activate",
+                     G_CALLBACK(gui_trigger_button_release), user);    
     g_signal_connect(G_OBJECT(button), "released",
                      G_CALLBACK(gui_trigger_button_release), user);    
 
@@ -486,6 +492,10 @@ GtkWidget *gui_trigger_create(USER *user)
     gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 2);
 
     button = gtk_button_new_with_label(BUTTON_APPLY);
+    gtk_button_set_use_underline(GTK_BUTTON(button) , TRUE);
+    /* activate is when the mnemonic key is pressed. */
+    g_signal_connect(G_OBJECT(button), "activate",
+                     G_CALLBACK(gui_trigger_button_release), user);    
     g_signal_connect(G_OBJECT(button), "released",
                      G_CALLBACK(gui_trigger_button_release), user);    
 
@@ -493,6 +503,10 @@ GtkWidget *gui_trigger_create(USER *user)
     gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 2);
 
     button = gtk_button_new_with_label(BUTTON_EXIT);
+    gtk_button_set_use_underline(GTK_BUTTON(button) , TRUE);
+    /* activate is when the mnemonic key is pressed. */
+    g_signal_connect(G_OBJECT(button), "activate",
+                     G_CALLBACK(gui_trigger_button_release), user);    
     g_signal_connect(G_OBJECT(button), "released",
                      G_CALLBACK(gui_trigger_button_release), user);    
 
