@@ -19,11 +19,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
+#if !defined(WIN32)
+  #include <unistd.h>
+#endif
 #include "mudix.h"
 
 static void menu_select(gpointer data, guint action, GtkWidget *widget);
 
+static char *wordDelimiters = " \"'`~!@#$%^&*()_-+=[]{}:;\\|/?.>,<";
 
 typedef enum
 {
@@ -35,49 +38,69 @@ typedef enum
 
 typedef enum
 {
-    MENU_NONE=0, FILE_OPEN, FILE_SAVE, FILE_EXIT, 
+    MENU_NONE, FILE_OPEN, FILE_SAVE, FILE_EXIT, 
     VIEW_GENERAL, VIEW_GENERAL_CHAR, VIEW_GENERAL_COLOR, VIEW_GENERAL_FONTS,
     VIEW_ALIASES, VIEW_MACROS, VIEW_PATHS, VIEW_TABS, VIEW_TIMERS, VIEW_TRIGGERS, VIEW_VARS,
+    WRAP_CHAR, WRAP_WORD, WRAP_NONE,
     SESSION_RECONNECT, SESSION_DISCONNECT, SESSION_TOGGLE_CLEAR, 
     SESSION_TOGGLE_ECHO, SESSION_TOGGLE_MUTE,
-    HELP_ABOUT
+    HELP_ABOUT,
+    EDIT_SELECTALL, EDIT_CUT, EDIT_COPY, EDIT_PASTE, EDIT_CUTWORD, EDIT_CUTLINE
 } MENU_ACTION;
 
 
 static GtkItemFactoryEntry menu_items[] =
 {
-  { "/File",                        NULL,           NULL,           MENU_NONE, "<Branch>" },
-  { "/File/Open User-Selector",     "<control>O",   menu_select,    FILE_OPEN },
-  { "/File/Save",                   "<control>S",   menu_select,    FILE_SAVE, "<StockItem>", GTK_STOCK_SAVE },
-  { "/File/sep1",                   NULL,           NULL,           MENU_NONE, "<Separator>" },
-  { "/File/Exit",                   "<control>X",   menu_select,    FILE_EXIT, "<StockItem>", GTK_STOCK_QUIT },
+  { "/File",                    NULL,                   NULL,           MENU_NONE, "<Branch>" },
+  { "/File/Open User-Selector", "<control>O",           menu_select,    FILE_OPEN },
+  { "/File/Save",               "<control>S",           menu_select,    FILE_SAVE, "<StockItem>", GTK_STOCK_SAVE },
+  { "/File/sep1",               NULL,                   NULL,           MENU_NONE, "<Separator>" },
+  { "/File/Exit",               "<control>Q",           menu_select,    FILE_EXIT, "<StockItem>", GTK_STOCK_QUIT },
+  { "/Edit",                    NULL,                   NULL,           MENU_NONE, "<Branch>" },
+  { "/Edit/Select All",         "<control>A",           menu_select,    EDIT_SELECTALL },
+  { "/Edit/sep1",               NULL,                   NULL,           MENU_NONE, "<Separator>" },
+  { "/Edit/Cut",                "<control>X",           menu_select,    EDIT_CUT },
+  { "/Edit/Copy",               "<control>C",           menu_select,    EDIT_COPY },
+  { "/Edit/Paste",              "<control>V",           menu_select,    EDIT_PASTE },
+  { "/Edit/sep1",               NULL,                   NULL,           MENU_NONE, "<Separator>" },
+  { "/Edit/Cut Word",           "<control>W",           menu_select,    EDIT_CUTWORD },
+  { "/Edit/Cut Line",           "<control>L",           menu_select,    EDIT_CUTLINE },
 
-  { "/View",                        NULL,           NULL,           MENU_NONE, "<Branch>" },
-  { "/View/Preferences",            "<control>G",   menu_select,    VIEW_GENERAL },
-  { "/View/sep1",                   NULL,           NULL,           MENU_NONE, "<Separator>" },
-  { "/View/General",                NULL,           NULL,           MENU_NONE, "<Branch>" },
-  { "/View/General/Character",      "<control>H",   menu_select,    VIEW_GENERAL_CHAR },
-  { "/View/General/Color",          "<control>C",   menu_select,    VIEW_GENERAL_COLOR },
-  { "/View/General/Fonts",          "<control>F",   menu_select,    VIEW_GENERAL_FONTS },
-  { "/View/sep1",                   NULL,           NULL,           MENU_NONE, "<Separator>" },
-  { "/View/Aliases",                "<control>A",   menu_select,    VIEW_ALIASES },
-  { "/View/Macros",                 "<control>M",   menu_select,    VIEW_MACROS },
-  { "/View/Paths",                  "<control>P",   menu_select,    VIEW_PATHS },
-  { "/View/Tab completion",         "<control>B",   menu_select,    VIEW_TABS },
-  { "/View/Timers",                 "<control>I",   menu_select,    VIEW_TIMERS },
-  { "/View/Triggers",               "<control>T",   menu_select,    VIEW_TRIGGERS },
-  { "/View/Variables",              "<control>V",   menu_select,    VIEW_VARS },
+  { "/View",                    NULL,                   NULL,           MENU_NONE, "<Branch>" },
+  { "/View/Preferences",        "<control>G",           menu_select,    VIEW_GENERAL },
+  { "/View/sep1",               NULL,                   NULL,           MENU_NONE, "<Separator>" },
+  { "/View/General",            NULL,                   NULL,           MENU_NONE, "<Branch>" },
+  { "/View/General/Character",  "<control><alt>H",      menu_select,    VIEW_GENERAL_CHAR },
+  { "/View/General/Color",      "<control><alt>C",      menu_select,    VIEW_GENERAL_COLOR },
+  { "/View/General/Fonts",      "<control><alt>F",      menu_select,    VIEW_GENERAL_FONTS },
+
+  /* NOTE: when the following wrapping paths are changed, do NOT forget to
+           update the corresponding paths at the end of the gui_setup_user_window()
+           function! */
+  { "/View/General/Wrapping",   NULL,                   NULL,           MENU_NONE, "<Branch>" },
+  { "/View/General/Wrapping/Character", NULL,           menu_select,    WRAP_CHAR, "<RadioItem>" },
+  { "/View/General/Wrapping/Word", NULL,                menu_select,    WRAP_WORD, "/View/General/Wrapping/Character" },
+  { "/View/General/Wrapping/None", NULL,                menu_select,    WRAP_NONE, "/View/General/Wrapping/Character" },
+
+  { "/View/sep1",               NULL,                   NULL,           MENU_NONE, "<Separator>" },
+  { "/View/Aliases",            "<control><alt>A",      menu_select,    VIEW_ALIASES },
+  { "/View/Macros",             "<control><alt>M",      menu_select,    VIEW_MACROS },
+  { "/View/Paths",              "<control><alt>P",      menu_select,    VIEW_PATHS },
+  { "/View/Tab completion",     "<control><alt>B",      menu_select,    VIEW_TABS },
+  { "/View/Timers",             "<control><alt>I",      menu_select,    VIEW_TIMERS },
+  { "/View/Triggers",           "<control><alt>T",      menu_select,    VIEW_TRIGGERS },
+  { "/View/Variables",          "<control><alt>V",      menu_select,    VIEW_VARS },
  
-  { "/Session",                     NULL,           NULL,           MENU_NONE, "<Branch>" },
-  { "/Session/Reconnect",           "<alt>R",       menu_select,    SESSION_RECONNECT },
-  { "/Session/Disconnect",          NULL,           menu_select,    SESSION_DISCONNECT },
-  { "/Session/sep1",                NULL,           NULL,           MENU_NONE, "<Separator>" },
-  { "/Session/Toggle command mute", "<alt>M",       menu_select,    SESSION_TOGGLE_CLEAR },
-  { "/Session/Toggle input echo",   "<alt>E",       menu_select,    SESSION_TOGGLE_ECHO },
-  { "/Session/Toggle input auto-clear", "<alt>C",   menu_select,    SESSION_TOGGLE_MUTE },
+  { "/Session",                         NULL,           NULL,           MENU_NONE, "<Branch>" },
+  { "/Session/Reconnect",               "<alt>R",       menu_select,    SESSION_RECONNECT },
+  { "/Session/Disconnect",              "<alt>D",       menu_select,    SESSION_DISCONNECT },
+  { "/Session/sep1",                    NULL,           NULL,           MENU_NONE, "<Separator>" },
+  { "/Session/Toggle command mute",     "<alt>M",       menu_select,    SESSION_TOGGLE_MUTE },
+  { "/Session/Toggle input echo",       "<alt>E",       menu_select,    SESSION_TOGGLE_ECHO },
+  { "/Session/Toggle input auto-clear", "<alt>C",       menu_select,    SESSION_TOGGLE_CLEAR },
 
-  { "/Help",                        NULL,           0,              MENU_NONE, "<Branch>" },
-  { "/Help/_About",                 NULL,           menu_select,    HELP_ABOUT },
+  { "/Help",                            NULL,           0,              MENU_NONE, "<LastBranch>" },
+  { "/Help/_About",                     NULL,           menu_select,    HELP_ABOUT },
 };
 
 
@@ -117,6 +140,37 @@ static void gui_about_box(USER *user)
 }
 
 
+static long getWordDelimiterOffset(gchar *input)
+{
+    gchar  *lastFind = (gchar*)0L, *curFind = (gchar*)0L;
+    gchar  *curLoc = input;
+    long    numWordDelimiters = strlen(wordDelimiters);
+    long    retVal = 0, i;
+    long    amountToSearch = g_utf8_strlen(input, -1);
+    gchar   gc;
+
+    for (i = 0; i < numWordDelimiters; i++)
+    {
+        gc = (gchar)wordDelimiters[i];
+        curFind = g_utf8_strrchr(curLoc, amountToSearch, gc);
+
+        if (curFind > lastFind)
+        {
+            lastFind = curFind;
+            curLoc = curFind;
+            amountToSearch = g_utf8_strlen(lastFind, -1);
+        }
+    }
+
+    if (lastFind != (gchar*)0L)
+    {
+        retVal = (long)lastFind - (long)input;
+    }
+
+    return(retVal);
+}
+
+
 static void menu_select(gpointer data, guint action, GtkWidget *widget)
 {
     USER *user;
@@ -133,7 +187,7 @@ static void menu_select(gpointer data, guint action, GtkWidget *widget)
             gui_setup_main_window();
             break;
         case FILE_EXIT:
-            gtk_widget_destroy(GTK_WIDGET(data));
+            destroy_with_signal(data);
             break;
         case FILE_SAVE:
             if (save_user(user))
@@ -178,6 +232,21 @@ static void menu_select(gpointer data, guint action, GtkWidget *widget)
         case VIEW_VARS:
             gui_preference_editor(user, SEL_VARS);
             break;
+        case WRAP_CHAR:
+            user->gui_user.wrap_mode = GTK_WRAP_CHAR;
+            gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(user->gui_user.g_view), 
+                                        user->gui_user.wrap_mode);
+            break;
+        case WRAP_WORD:
+            user->gui_user.wrap_mode = GTK_WRAP_WORD;
+            gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(user->gui_user.g_view), 
+                                        user->gui_user.wrap_mode);
+            break;
+        case WRAP_NONE:
+            user->gui_user.wrap_mode = GTK_WRAP_NONE;
+            gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(user->gui_user.g_view), 
+                                        user->gui_user.wrap_mode);
+            break;
         case SESSION_RECONNECT:
             /* just call the reconnect function */
             gui_user_reconnect(user);
@@ -198,6 +267,44 @@ static void menu_select(gpointer data, guint action, GtkWidget *widget)
         case HELP_ABOUT:
             gui_about_box(user);
             break;
+        case EDIT_SELECTALL:
+            gtk_editable_select_region(GTK_EDITABLE(user->gui_user.g_input), 0, -1);
+            break;
+        case EDIT_CUTWORD:
+            {
+                long    start = 0;
+                gchar  *input;
+
+                input = gtk_editable_get_chars(GTK_EDITABLE(user->gui_user.g_input), start, -1);
+                start = getWordDelimiterOffset(input);
+
+                gtk_editable_delete_text(GTK_EDITABLE(user->gui_user.g_input), start, -1);
+            }
+            break;
+        case EDIT_CUTLINE:
+            gtk_editable_delete_text(GTK_EDITABLE(user->gui_user.g_input), 0, -1);
+            break;
+        case EDIT_COPY:
+            {
+              gint start, end;
+
+              if (gtk_editable_get_selection_bounds(GTK_EDITABLE(user->gui_user.g_input),
+                                                    &start, &end))
+              {
+                  gtk_editable_copy_clipboard(GTK_EDITABLE(user->gui_user.g_input));
+              }
+              else
+              {
+                  g_signal_emit_by_name(user->gui_user.g_view, "copy-clipboard");
+              }
+            }
+            break;
+        case EDIT_CUT:
+            gtk_editable_cut_clipboard(GTK_EDITABLE(user->gui_user.g_input));
+            break;
+        case EDIT_PASTE:
+            gtk_editable_paste_clipboard(GTK_EDITABLE(user->gui_user.g_input));
+            break;
         default:
             break;
     }
@@ -400,7 +507,7 @@ static void gui_set_view_window(USER *user, GtkWidget *view)
     /* change attributes of the view */
     gtk_text_view_set_editable(GTK_TEXT_VIEW(view), FALSE);
     gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(view), FALSE);
-    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(view), GTK_WRAP_CHAR);
+    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(view), user->gui_user.wrap_mode);
 
     /* update the FG and BG color in the view */
     gui_color_def_widget(user, view);
@@ -414,12 +521,20 @@ static gboolean gui_user_config_event(GtkWidget         *widget,
     /* just store our new size so we can save it later */
     if (widget == user->gui_user.g_window)
     {
-        user->gui_user.win_height = event->height;
-        user->gui_user.win_width  = event->width;
+        if (event->height != user->gui_user.win_height ||
+            event->width  != user->gui_user.win_width)
+        {
+            user->gui_user.win_height = event->height;
+            user->gui_user.win_width  = event->width;
+
+            /* size got updated, send NAWS if required */
+            send_naws(user);
+        }
+
+        user->gui_user.win_pos_x = event->x;
+        user->gui_user.win_pos_y = event->y;
     }
 
-    /* size got updated, send NAWS if required */
-    send_naws(user);
 
     /* FALSE passes the event through into the widget */
     return FALSE;
@@ -516,7 +631,7 @@ static void disconnect_response(GtkDialog *dialog, DIA_RESPONSE resp, USER *user
             break;
         case DIA_RESP_CLOSE:
             /* destroy the user window */
-            gtk_widget_destroy(user->gui_user.g_window);
+            destroy_with_signal(user->gui_user.g_window);
             break;
         case DIA_RESP_OFFLINE:
         default:
@@ -853,9 +968,9 @@ void gui_add_color_window(USER *user, gchar *buf, gsize len, guint color)
 void gui_add_ansi_window(USER *user, gchar *buf, gsize len)
 {
     GtkTextIter start;
-    guchar      fg = user->gui_user.ansi.fg;
-    guchar      bg = user->gui_user.ansi.bg;
-    guint       attrib = user->gui_user.ansi.attrib;
+    guchar      fg = user->ansi.fg;
+    guchar      bg = user->ansi.bg;
+    guint       attrib = user->ansi.attrib;
 
     /* insert text */
     gtk_text_buffer_insert(user->gui_user.g_buffer, 
@@ -901,7 +1016,7 @@ void gui_add_ansi_window(USER *user, gchar *buf, gsize len)
 
     if (attrib & ATTR_ITALIC)
     {
-        /* underline the text */
+        /* make the text italic */
         gtk_text_buffer_apply_tag(user->gui_user.g_buffer,
                                   user->gui_user.g_italic_tag,
                                   &start,
@@ -936,6 +1051,20 @@ void gui_add_ansi_window(USER *user, gchar *buf, gsize len)
 }
 
 
+void gui_user_cls(USER *user)
+{
+    GtkTextIter start, end;
+
+    gtk_text_buffer_get_start_iter(user->gui_user.g_buffer, &start);
+    gtk_text_buffer_get_end_iter(user->gui_user.g_buffer, &end);
+    gtk_text_buffer_delete(user->gui_user.g_buffer, &start, &end);
+
+    /* important, update the input iter because delete also deletes iters */
+    gtk_text_buffer_get_end_iter(user->gui_user.g_buffer, 
+                                 &user->gui_user.input_iter);
+}
+
+
 void gui_user_setup_tags(USER *user)
 {
     int i;
@@ -988,6 +1117,7 @@ void gui_setup_user_window(USER *user)
     GtkTextMark            *mark_begin, *mark_end;
     GtkWidget              *scrolled_window;
     GtkWidget              *vbox;
+    GtkWidget              *combo;
     GtkWidget              *entry;
     GtkWidget              *statusbar;
     GtkWidget              *vpaned;
@@ -1000,11 +1130,11 @@ void gui_setup_user_window(USER *user)
 
     /* setup event handlers for the window */
     g_signal_connect(G_OBJECT(window), "destroy",
-	             G_CALLBACK(gui_destroy_window_callback), user);
+                     G_CALLBACK(gui_destroy_window_callback), user);
     g_signal_connect(G_OBJECT(window), "configure-event",
-		     G_CALLBACK(gui_user_config_event), user);
+                     G_CALLBACK(gui_user_config_event), user);
     g_signal_connect(G_OBJECT(window), "key-press-event",
-		     G_CALLBACK(gui_key_event_callback), user);
+                     G_CALLBACK(gui_key_event_callback), user);
 
     gtk_container_set_border_width(GTK_CONTAINER(window), 2);
     gtk_window_set_default_size(GTK_WINDOW(window),
@@ -1029,10 +1159,9 @@ void gui_setup_user_window(USER *user)
                            item_factory, (GDestroyNotify)g_object_unref);
 
     /* create menu items */
-    gtk_item_factory_create_items(item_factory, G_N_ELEMENTS(menu_items),
-                                  menu_items, window);
+    gtk_item_factory_create_items(item_factory, G_N_ELEMENTS(menu_items), menu_items, window);
 
-    /* put the menu in the vertical box */
+     /* put the menu in the vertical box */
     gtk_box_pack_start(GTK_BOX(vbox), 
                        gtk_item_factory_get_widget(item_factory, "<main>"), 
                        FALSE, FALSE, 1);
@@ -1050,8 +1179,8 @@ void gui_setup_user_window(USER *user)
     /* create a scrolled window */
     scrolled_window = gtk_scrolled_window_new(NULL, NULL);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
-		   	           GTK_POLICY_AUTOMATIC,
-				   GTK_POLICY_ALWAYS);
+                                   GTK_POLICY_AUTOMATIC,
+                                   GTK_POLICY_ALWAYS);
     /* put the text view inside the scrolled window */
     gtk_container_add(GTK_CONTAINER(scrolled_window), view);
 
@@ -1070,8 +1199,8 @@ void gui_setup_user_window(USER *user)
     /* create a scrolled window */
     scrolled_window = gtk_scrolled_window_new(NULL, NULL);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
-		   	           GTK_POLICY_AUTOMATIC,
-				   GTK_POLICY_ALWAYS);
+                                   GTK_POLICY_AUTOMATIC,
+                                   GTK_POLICY_ALWAYS);
     /* put the text view inside the scrolled window */
     gtk_container_add(GTK_CONTAINER(scrolled_window), scrollback);
 
@@ -1092,19 +1221,21 @@ void gui_setup_user_window(USER *user)
     mark_end = gtk_text_buffer_create_mark(buffer, NULL, &iter, FALSE);
 
     /* create an entry widget */
-    entry = gtk_entry_new();
+    combo = gtk_combo_new();
+    entry = GTK_COMBO(combo)->entry;
+    gtk_combo_disable_activate(GTK_COMBO(combo));
     gtk_entry_set_max_length(GTK_ENTRY(entry), MAX_INPUT_LEN-1);
 
     /* set up event handlers for the input bar */
     g_signal_connect(G_OBJECT(entry), "activate",
-		     G_CALLBACK(gui_enter_callback), user);
+                     G_CALLBACK(gui_enter_callback), user);
     g_signal_connect(G_OBJECT(entry), "focus-in-event",
-		     G_CALLBACK(gui_user_focus_in), user);
+                     G_CALLBACK(gui_user_focus_in), user);
     g_signal_connect(G_OBJECT(entry), "focus-out-event",
-		     G_CALLBACK(gui_user_focus_out), user);
+                     G_CALLBACK(gui_user_focus_out), user);
 
     /* also put the entry inside the vertical box */
-    gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 1);
+    gtk_box_pack_start(GTK_BOX(vbox), combo, FALSE, FALSE, 1);
 
     /* create the statusbar */
     statusbar = gtk_statusbar_new();
@@ -1115,16 +1246,15 @@ void gui_setup_user_window(USER *user)
     /* get the end iter which will point to the insertion point */
     gtk_text_buffer_get_end_iter(buffer, &user->gui_user.input_iter);
 
-    /* set up some usable pointers in the user structure */
-    user->gui_user.g_window      = window;
-    user->gui_user.g_view        = view;
-    user->gui_user.g_buffer      = buffer;
-    user->gui_user.g_input       = entry;
-    user->gui_user.g_status      = statusbar;
-    user->gui_user.g_mark        = mark_end;
-
-    /* NOTE! g_scrollback is the scrolled_window and not the view */
-    user->gui_user.g_scrollback  = GTK_PANED(vpaned)->child1;
+    /* set up some useful pointers in the user structure */
+    user->gui_user.g_window     = window;
+    user->gui_user.g_view       = view;
+    user->gui_user.g_buffer     = buffer;
+    user->gui_user.g_input      = entry;
+    user->gui_user.g_combo      = combo;
+    user->gui_user.g_status     = statusbar;
+    user->gui_user.g_mark       = mark_end;
+    user->gui_user.g_scrollback = GTK_PANED(vpaned)->child1;
 
     /* set the focus to the entry bar only */
     g_object_set(view, "can-focus", FALSE, NULL);
@@ -1136,13 +1266,46 @@ void gui_setup_user_window(USER *user)
     gui_font_update(user, FONT_USER_WINDOW);
     gui_font_update(user, FONT_USER_SCROLLBACK);
     gui_font_update(user, FONT_USER_INPUT);
+    gui_font_spacing_update(user);
 
     /* move window to saved position */
     if (user->gui_user.win_pos_x && user->gui_user.win_pos_y)
     {
-        gtk_window_move(GTK_WINDOW(user->gui_user.g_window),
-                        user->gui_user.win_pos_x,
-                        user->gui_user.win_pos_y);
+        int x = user->gui_user.win_pos_x - X_COMPENSATION;
+        int y = user->gui_user.win_pos_y - Y_COMPENSATION;
+
+        if (x < 0)
+        {
+            x = 0;
+        }
+
+        if (y < 0)
+        {
+            y = 0;
+        }
+
+        gtk_window_move(GTK_WINDOW(user->gui_user.g_window), x, y);
+    }
+
+    /* initially set the menu radio button for wrap mode */
+    /* NOTE: This has to be called at this location or else error messages are generated
+       as soon as the user window is created (bug in GTK?) */
+    switch (user->gui_user.wrap_mode)
+    {
+        case GTK_WRAP_NONE:
+            gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(
+                gtk_item_factory_get_item(item_factory, "/View/General/Wrapping/None")), TRUE);
+            break;
+        case GTK_WRAP_CHAR:
+            gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(
+                gtk_item_factory_get_item(item_factory, "/View/General/Wrapping/Character")), TRUE);
+            break;
+        case GTK_WRAP_WORD:
+            gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(
+                gtk_item_factory_get_item(item_factory, "/View/General/Wrapping/Word")), TRUE);
+            break;
+          default:
+            break;
     }
 
     /* set the title of the window */
