@@ -20,7 +20,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <arpa/telnet.h>
+#if !defined(WIN32)
+  #include <arpa/telnet.h>
+#else
+  #include <windows.h>
+#endif
 #include "mudix.h"
 
 
@@ -111,12 +115,12 @@ static bool check_expression(USER *user, gchar *expr)
     pArg1 = arg1;
     while ((c = g_utf8_get_char(pExpr)) != '\0')
     {
-	if (c == '!' || 
+        if (c == '!' || 
             c == '=' ||
             c == '>' ||
             c == '<')
         {
-	    break;
+            break;
         }
 
         pArg1 += g_unichar_to_utf8(c, pArg1);
@@ -217,7 +221,7 @@ bool process_if(USER *user, gchar *buffer)
         /* grab first argument after expression and process it */
         get_arg(user, buffer, str);
         process_input(user, str, NULL);
-	return TRUE;
+        return TRUE;
     }
     else
     {
@@ -370,7 +374,7 @@ gchar *parse_input(USER *user, gchar *src, gchar *args, gchar *dest, int *len)
             else if (g_unichar_isdigit(var))
             {
                 /* enough arguments in table for requested arg? */
-                if (arg_count > (var - '0'))
+                if (arg_count > (int)(var - '0'))
                 {
                     gchar *pTmp;
 
@@ -410,7 +414,7 @@ gchar *parse_input(USER *user, gchar *src, gchar *args, gchar *dest, int *len)
                     pTmp = var->value;
                     while (*pTmp != '\0')
                     {
-	                *pDest++ = *pTmp++;
+                        *pDest++ = *pTmp++;
 
                         /* safety check */
                         if ((pDest - dest) >= MAX_STRING)
@@ -453,20 +457,35 @@ void process_input(USER *user, gchar *input, gchar *args)
 
     do
     {
-        /* parse the input */
-	input = parse_input(user, input, args, cmd, &len);
+        gunichar first_char;
 
-        /* is it a MUDix-command? */
-        if (g_utf8_get_char(cmd) == user->custom_chars[CUST_CMD_CHAR] &&
-            do_command(user, g_utf8_next_char(cmd)))
+        /* parse the input */
+        input = parse_input(user, input, args, cmd, &len);
+
+        /* grab the first character */
+        first_char = g_utf8_get_char(cmd);
+
+        if (first_char == user->custom_chars[CUST_CMD_CHAR])
         {
-            continue;
+            /* check if a MUDix-command is to be processed */
+            if (do_command(user, g_utf8_next_char(cmd)))
+            {
+                continue;
+            }
+        }
+        else if (first_char == user->custom_chars[CUST_SPEED_PATH])
+        {
+            /* check if a speed path is to be processed */
+            if (process_path(user, g_utf8_next_char(cmd)))
+            {
+                continue;
+            }
         }
 
         /* check if an alias needs to be processed */
-    	if (process_alias(user, cmd))
+        if (process_alias(user, cmd))
         {
-	    continue;
+            continue;
         }
 
         /* write the data to the socket */
@@ -484,147 +503,175 @@ void process_input(USER *user, gchar *input, gchar *args)
 }
 
 
-static bool check_esc(gunichar start, gchar **pRx, gchar *pEnd, ANSI_T *ansi)
+static void process_esc(gunichar c, ANSI_T *ansi)
 {
-    gchar        local[5];
-    gchar       *pLoc = local;
-    gchar       *pBuf = *pRx;
-    int          params[CSIPARAMS], npar, i;
-    gunichar     c;
-    bool         fParams = FALSE;
+    char     *pParam;
+    int       params[CSIPARAMS], npar = 0, i;
 
-    if (start == CSI)
+    *ansi->paramp = '\0';
+    pParam        = ansi->parambuf;
+    while (pParam < ansi->paramp)
     {
-        fParams = TRUE;
+        params[npar++] = atoi(pParam);
+        pParam        += (strlen(pParam) + 1);
     }
 
-    /* first skip the ESC/CSI sign */
-    pBuf = g_utf8_next_char(pBuf);
-
-    npar = 0;
-    while (pBuf < pEnd) 
+    switch (c) 
     {
-        /* grab unichar */
-        c = g_utf8_get_char(pBuf);
+        /* we only support 'm' for now */
+        case 'm':
+            /* if no parameters are found, just clear */
+            if (npar == 0)
+            {
+                ansi->attrib = 0;
+                ansi->fg     = ANSI_COLOR_NONE;
+                ansi->bg     = ANSI_COLOR_NONE;
+            }
 
-        if (c == '[')
-        {
-            /* start of parameter character */
-            fParams = TRUE;
-        }
-        else if (g_unichar_isdigit(c) && fParams)
-        {
-            /* store character if number within range of 4 digits */
-            if ((pLoc - local) < 5)
+            for (i=0; i<npar; i++) 
             {
-	        *pLoc++ = (char)c;
+                 switch (params[i]) 
+                 {
+                     case ANSI_DEFAULT:
+                         ansi->attrib = 0;
+                         ansi->fg     = ANSI_COLOR_NONE;
+                         ansi->bg     = ANSI_COLOR_NONE;
+                         break;
+                     case ANSI_BOLD:
+                         ansi->attrib |= ATTR_BOLD;
+                         break;
+                     case ANSI_UNDERLINE:
+                         ansi->attrib |= ATTR_UNDERLINE;
+                         break;
+                     case ANSI_ITALIC:
+                         ansi->attrib |= ATTR_ITALIC;
+                         break;
+                     case ANSI_BOLD_OFF:
+                         ansi->attrib &= ~ATTR_BOLD;
+                         break;
+                     case ANSI_UNDERLINE_OFF:
+                         ansi->attrib &= ~ATTR_UNDERLINE;
+                         break;
+                     case ANSI_ITALIC_OFF:
+                         ansi->attrib &= ~ATTR_ITALIC;
+                         break;
+                     case ANSI_FG_BLACK:
+                     case ANSI_FG_RED:
+                     case ANSI_FG_GREEN:
+                     case ANSI_FG_BROWN:
+                     case ANSI_FG_BLUE:
+                     case ANSI_FG_MAGENTA:
+                     case ANSI_FG_CYAN:
+                     case ANSI_FG_WHITE:
+                         /* set fg color */
+                         ansi->fg = (params[i] - ANSI_FG_BLACK);
+                         break;
+                     case ANSI_BG_BLACK:
+                     case ANSI_BG_RED:
+                     case ANSI_BG_GREEN:
+                     case ANSI_BG_BROWN:
+                     case ANSI_BG_BLUE:
+                     case ANSI_BG_MAGENTA:
+                     case ANSI_BG_CYAN:
+                     case ANSI_BG_WHITE:
+                         /* set bg color */
+                         ansi->bg = (params[i] - ANSI_BG_BLACK);
+                         break;
+                     case ANSI_FG_DEFAULT:
+                         ansi->fg = 0;
+                         break;
+                     case ANSI_BG_DEFAULT:
+                         ansi->bg = 0;
+                         break;
+                     default:
+                         break;
+                 }
             }
-        }
-        else if (c == ';')
+            break;
+
+        default:
+            break;
+    }
+
+    /* reset the ansi state and rx pointer */
+    ansi->state  = ANSI_STATE_IDLE;
+    ansi->paramp = ansi->parambuf;
+}
+                  
+
+static guint check_esc(gunichar c, gchar *pRx, gchar *pEnd, ANSI_T *ansi)
+{
+    gchar *pStart = pRx;
+
+    while (pRx < pEnd) 
+    {
+        switch (ansi->state)
         {
-            /* end of parameter */
-            if (pLoc != local && npar < CSIPARAMS)
-            {
-                *pLoc = '\0';
-	        params[npar++] = atoi(local); 
-                pLoc = local;
-            }
+            case ANSI_STATE_IDLE:
+                if (c == CSI)
+                {
+                    ansi->state = ANSI_STATE_PARAMS;
+                }
+                else if (c == ESC)
+                {
+                    ansi->state = ANSI_STATE_ESC;
+                }
+                else
+                {
+                    /* do nothing */
+                }
+                break;
+
+            case ANSI_STATE_ESC:
+                if (c == '[')
+                {
+                    ansi->state = ANSI_STATE_PARAMS;
+                }
+                else
+                {
+                    /* process the complete esc seq */
+                    process_esc(c, ansi);
+                }
+                break;
+
+            case ANSI_STATE_PARAMS:
+                if (c == ';')
+                {
+                    *ansi->paramp++ = '\0';
+                    // overflow check here?
+                }
+                else if (g_unichar_isdigit(c))
+                {
+                    *ansi->paramp++ = (char)c;
+                    // overflow check here?
+                }
+                else
+                {
+                    /* process the complete esc seq */
+                    process_esc(c, ansi);
+                }
+                break;
+
+            default:
+                /* unknown state? */
+                break;
+        }
+
+        /* advance pointer */
+        pRx = g_utf8_next_char(pRx);
+
+        if (ansi->state == ANSI_STATE_IDLE)
+        {
+            break;
         }
         else
         {
-            /* everything else is end of escape sequence */
-            if (pLoc != local && npar < CSIPARAMS)
-            {
-                *pLoc = '\0';
-	        params[npar++] = atoi(local); 
-                pLoc = local;
-            }
-
-            switch (c) 
-            {
-	        case 'm':
-                    /* if no parameters are found, just clear */
-                    if (npar == 0)
-                    {
-                        ansi->attrib = 0;
-                        ansi->fg     = ANSI_COLOR_NONE;
-                        ansi->bg     = ANSI_COLOR_NONE;
-                    }
-
-                    for (i=0; i<npar; i++) 
-                    {
-                        switch (params[i]) 
-                        {
-                            case ANSI_DEFAULT:
-                                ansi->attrib = 0;
-                                ansi->fg     = ANSI_COLOR_NONE;
-                                ansi->bg     = ANSI_COLOR_NONE;
-                                break;
-                            case ANSI_BOLD:
-                                ansi->attrib |= ATTR_BOLD;
-                                break;
-                            case ANSI_UNDERLINE:
-                                ansi->attrib |= ATTR_UNDERLINE;
-                                break;
-                            case ANSI_ITALIC:
-                                ansi->attrib |= ATTR_ITALIC;
-                                break;
-                            case ANSI_BOLD_OFF:
-                                ansi->attrib &= ~ATTR_BOLD;
-                                break;
-                            case ANSI_UNDERLINE_OFF:
-                                ansi->attrib &= ~ATTR_UNDERLINE;
-                                break;
-                            case ANSI_ITALIC_OFF:
-                                ansi->attrib &= ~ATTR_ITALIC;
-                                break;
-                            case ANSI_FG_BLACK:
-                            case ANSI_FG_RED:
-                            case ANSI_FG_GREEN:
-                            case ANSI_FG_BROWN:
-                            case ANSI_FG_BLUE:
-                            case ANSI_FG_MAGENTA:
-                            case ANSI_FG_CYAN:
-                            case ANSI_FG_WHITE:
-                                /* set fg color */
-                                ansi->fg = (params[i] - ANSI_FG_BLACK);
-                                break;
-                            case ANSI_BG_BLACK:
-                            case ANSI_BG_RED:
-                            case ANSI_BG_GREEN:
-                            case ANSI_BG_BROWN:
-                            case ANSI_BG_BLUE:
-                            case ANSI_BG_MAGENTA:
-                            case ANSI_BG_CYAN:
-                            case ANSI_BG_WHITE:
-                                /* set bg color */
-                                ansi->bg = (params[i] - ANSI_BG_BLACK);
-                                break;
-                            case ANSI_FG_DEFAULT:
-                                ansi->fg = 0;
-                                break;
-                            case ANSI_BG_DEFAULT:
-                                ansi->bg = 0;
-                                break;
-                            default:
-                                break;
-                        }
-                    }
-
-                    break;
-                default:
-                    break;
-            } /* end switch */
-
-            /* return the new pointer */
-            *pRx = g_utf8_next_char(pBuf);
-            return TRUE;
+            /* get next character */
+            c = g_utf8_get_char(pRx);
         }
-
-        pBuf = g_utf8_next_char(pBuf);
     }
  
-    return FALSE;   /* push the current escape sequence back for later check */
+    return pRx-pStart;
 }
 
 
@@ -650,7 +697,7 @@ static void add_to_trigger_buf(USER *user, gchar *buf, int len)
 static void process_rx_data(USER *user, char *data, int len)
 {
     GError   *conv_err = NULL;
-    gchar    *rxbuf, *pRx, *pMrk;
+    gchar    *rxbuf, *pRx, *pMrk, *pEnd;
     gsize     rxcnt;
     gunichar  c;
 
@@ -671,139 +718,118 @@ static void process_rx_data(USER *user, char *data, int len)
         return;
     }
 
-    pRx = pMrk = rxbuf;
+    pRx  = pMrk = rxbuf;
+    pEnd        = rxbuf + rxcnt;
+
     /* check all characters in the buffer */
-    while ((pRx-rxbuf) < rxcnt)
+    while (pRx != pEnd)
     {
-        switch ((c = g_utf8_get_char(pRx)))
+        c = g_utf8_get_char(pRx);
+
+        if (user->ansi.state != ANSI_STATE_IDLE)
         {
-            case CSI:
-            case ESC:
-                /* found an escape sequence, flush buffer and check ESC */
-                if (pRx != pMrk) 
-                {
-                    gui_add_ansi_window(user, pMrk, pRx-pMrk);
-                    add_to_trigger_buf(user, pMrk, pRx-pMrk);
-                }
-
-                if (!check_esc(c, &pRx, rxbuf + rxcnt, &user->gui_user.ansi))
-                {
-                    /* only received part of an ESC sequence! */
-                    gchar *esc;
-                    gsize  len;
-
-                    /* convert the rest of the UTF8 buffer back to the charset */
-                    esc = g_convert(pRx, rxcnt - (pRx-rxbuf), user->net.charset, 
-                                    CHARSET_CONV, NULL, &len, &conv_err);
-
-                    /* check if conversion resulted in an error */
-                    if (!esc)
-                    {
-                        /* print errors */
-                        fprintf(stderr, "push_esc: conv_err (%d): %s\n", 
-                                conv_err->code, conv_err->message);
-
-                        /* free the error structure */
-                        g_error_free(conv_err);
-                    }
-                    else
-                    {
-                        /* push the ESC sequence back into the rx buffer */
-                        if (MCCP_USER(user))
-                        {
-                            memcpy(user->net.mccp_out, esc, len);
-                            user->net.mccp_outp = user->net.mccp_out + len;
-                        }
-                        else
-                        {
-                            memcpy(user->net.rxbuf, esc, len);
- 	                    user->net.rxp = user->net.rxbuf + len;
-                        }
-
-                        /* finally free the temporary conversion buffer */
-                        g_free(esc);
-                    }
-
-                    /* indicates we reached end of the pointer */
-                    pRx = rxbuf + rxcnt;
-                }
-
-                pMrk = pRx;
-                break;
-            case LF:
-                /* encountered a newline, first add data (without newline) */
-                if (pRx != pMrk) 
-                {
-                    add_to_trigger_buf(user, pMrk, pRx-pMrk);
-                }
-
-                /* also grab the newline */
-                pRx = g_utf8_next_char(pRx);
-
-                /* newline is now also sent to user window */
-                if (pRx != pMrk) 
-                {
-                    gui_add_ansi_window(user, pMrk, pRx-pMrk);
-                }
-
-                /* update the marker to the new position */
-                pMrk = pRx;
-
-                /* finally check if this line is triggered */
-	        trigger_check(user, TRUE);
-                break;
-            case BS:
-                /* encountered a backspace, flush buffer and skip it */
-                if (pRx != pMrk) 
-                {
-                    /* do not print the last character in the buffer */
-                    gchar *pTmp = g_utf8_prev_char(pRx);
-
-                    gui_add_ansi_window(user, pMrk, pTmp-pMrk);
-                    add_to_trigger_buf(user, pMrk, pTmp-pMrk);
-                }
-                else
-                {
-                    /* delete the last character */
-                    gui_user_backspace(user);
-                }
-
-                /* skip the BS */
-                pMrk = pRx = g_utf8_next_char(pRx);
-                break;
-            case BEEP:
-                /* encountered a beep, flush buffer and skip it */
-                if (pRx != pMrk) 
-                {
-                    gui_add_ansi_window(user, pMrk, pRx-pMrk);
-                    add_to_trigger_buf(user, pMrk, pRx-pMrk);
-                }
-
-                /* sound the beep :) */
-                gdk_beep();
-
-                /* skip the BEEP */
-                pMrk = pRx = g_utf8_next_char(pRx);
-                break;
-            default:
-                if (!g_unichar_isprint(c))
-                {
-                    /* unsupported character, flush buffer */
+            pRx += check_esc(c, pRx, pEnd, &user->ansi);
+            pMrk = pRx;
+        }
+        else
+        {
+            switch (c)
+            {
+                case CSI:
+                case ESC:
+                    /* found an escape sequence, flush buffer and check ESC */
                     if (pRx != pMrk) 
                     {
                         gui_add_ansi_window(user, pMrk, pRx-pMrk);
                         add_to_trigger_buf(user, pMrk, pRx-pMrk);
                     }
 
-                    /* skip it */
-                    pMrk = pRx = g_utf8_next_char(pRx);
-                }
-                else
-                {
-                    /* just continue with next character */
+                    pRx += check_esc(c, pRx, pEnd, &user->ansi);
+                    pMrk = pRx;
+                    break;
+
+                case LF:
+                    /* encountered a newline, first add data (without newline) */
+                    if (pRx != pMrk) 
+                    {
+                        add_to_trigger_buf(user, pMrk, pRx-pMrk);
+                    }
+
+                    /* also grab the newline */
                     pRx = g_utf8_next_char(pRx);
-                }
-                break;
+
+                    /* newline is now also sent to user window */
+                    if (pRx != pMrk) 
+                    {
+                        gui_add_ansi_window(user, pMrk, pRx-pMrk);
+                    }
+
+                    /* update the marker to the new position */
+                    pMrk = pRx;
+
+                    /* finally check if this line is triggered */
+                    trigger_check(user, TRUE);
+                    break;
+
+                case BS:
+                    /* encountered a backspace, flush buffer and skip it */
+                    if (pRx != pMrk) 
+                    {
+                        /* do not print the last character in the buffer */
+                        gchar *pTmp = g_utf8_prev_char(pRx);
+
+                        gui_add_ansi_window(user, pMrk, pTmp-pMrk);
+                        add_to_trigger_buf(user, pMrk, pTmp-pMrk);
+                    }
+                    else
+                    {
+                        /* delete the last character */
+                        gui_user_backspace(user);
+                    }
+
+                    /* skip the BS */
+                    pMrk = pRx = g_utf8_next_char(pRx);
+                    break;
+
+                case BEEP:
+                    /* encountered a beep, flush buffer and skip it */
+                    if (pRx != pMrk) 
+                    {
+                        gui_add_ansi_window(user, pMrk, pRx-pMrk);
+                        add_to_trigger_buf(user, pMrk, pRx-pMrk);
+                    }
+
+                    /* sound the beep :) */
+#if !defined(WIN32)
+                    gdk_beep();
+#else
+                    MessageBeep(MB_OK);
+#endif
+
+                    /* skip the BEEP */
+                    pMrk = pRx = g_utf8_next_char(pRx);
+                    break;
+
+                default:
+                    if (!g_unichar_isprint(c))
+                    {
+                        /* unsupported character, flush buffer */
+                        if (pRx != pMrk) 
+                        {
+                            gui_add_ansi_window(user, pMrk, pRx-pMrk);
+                            add_to_trigger_buf(user, pMrk, pRx-pMrk);
+                        }
+
+                        /* skip it */
+                        pMrk = pRx = g_utf8_next_char(pRx);
+                    }
+                    else
+                    {
+                        /* just continue with next character */
+                        pRx = g_utf8_next_char(pRx);
+                    }
+                    break;
+            }
         }
     }
 
@@ -817,7 +843,7 @@ static void process_rx_data(USER *user, char *data, int len)
     /* finally check if this line is triggered */
     trigger_check(user, FALSE);
 
-    /* free the remporary conversion buffer */
+    /* free the temporary conversion buffer */
     g_free(rxbuf);
 
     return;
@@ -826,8 +852,7 @@ static void process_rx_data(USER *user, char *data, int len)
 
 void process_rx_buffer(USER *user) 
 {
-    unsigned char *pRx, *pMrk, *pEnd;
-             int   retval = IAC_OK;
+    unsigned char *pRx, *pMrk1, *pMrk2, *pEnd;
 
     /* check if the MCCP stream is allocated */
     if (MCCP_USER(user))
@@ -836,102 +861,113 @@ void process_rx_buffer(USER *user)
         decompress_rxbuf(user);
 
         /* set pointers to the MCCP buffer */
-        pRx = pMrk = user->net.mccp_out;
-        pEnd = user->net.mccp_outp;
+        pRx  = pMrk1 = pMrk2 = user->net.mccp_out;
+        pEnd                 = user->net.mccp_outp;
     }
     else
     {
         /* set pointers to the receive buffers */
-        pRx  = pMrk = user->net.rxbuf;
-        pEnd = user->net.rxp;
+        pRx  = pMrk1 = pMrk2 = user->net.rxbuf;
+        pEnd                 = user->net.rxp;
     }
 
     /* look for IAC's */
     while (pRx != pEnd)
     {
-        if (*pRx == IAC)
+        if (user->net.rx_proc_state == RX_PROC_STATE_DATA)
         {
-            retval = check_iac(user, pRx, pEnd);
-
-            /* first process the data that is still there */
-            if (pRx != pMrk)
+            if (*pRx++ == IAC)
             {
-                process_rx_data(user, pMrk, pRx-pMrk);
-            }
-
-            if (retval == IAC_ESCAPED)
-            {
-                /* skip one IAC */
-                pMrk = (pRx+1);
-                pRx += IAC_ESCAPED_SIZE;
-            }
-            else if (retval == IAC_BUFFER_END)
-            {
-                /* push it back in receive buffer - abuse pMrk to copy */
-                pMrk = MCCP_USER(user)? user->net.mccp_out: 
-                                        user->net.rxbuf;
-                while (pRx != pEnd)
-                {
-                    *pMrk++ = *pRx++;
-                }
-
-                /* re-set receive pointer */
-                MCCP_USER(user)? user->net.mccp_outp: 
-                                 user->net.rxp = pMrk;
-                break;
-            }
-            else if (retval == IAC_MCCP_START)
-            {
-                /* skip the IAC - we know the size */
-                pRx += IAC_MCCP_START_SIZE;
-
-                /* move the binary data after this IAC */
-                if (pRx < pEnd)
-                {
-                    memmove(user->net.rxbuf, pRx, pEnd-pRx);
-                    user->net.rxp = user->net.rxbuf + (pEnd-pRx);
-
-                    /* re-call ourselves */
-                    process_rx_buffer(user);
-                }
-                else
-                {
-                    user->net.rxp = user->net.rxbuf;
-                }
-                return;
+                /* set the state to process IAC data */
+                user->net.rx_proc_state = RX_PROC_STATE_IAC;
             }
             else
             {
-                /* update the pointers */
-                pRx += retval;
-                pMrk = pRx;
+                /* second marker follows pRx if non-IAC */
+                pMrk2 = pRx;
             }
         }
         else
         {
-            /* check next character */
-            pRx++;
+            /* currently processing IAC data */
+            int iac_length, iac_retval;
+
+            iac_retval = check_iac(user, pRx, pEnd, &iac_length);
+
+            switch (iac_retval)
+            {
+                case IAC_ESCAPED:
+                    /* restore the state */
+                    user->net.rx_proc_state = RX_PROC_STATE_DATA;
+
+                    /* skip one IAC */
+                    pMrk2 = pRx++;
+                    break;
+
+                case IAC_MCCP_START:
+                    /* restore the state */
+                    user->net.rx_proc_state = RX_PROC_STATE_DATA;
+
+                    /* skip the complete IAC */
+                    pRx += iac_length;
+
+                    if (pMrk1 != pMrk2)
+                    {
+                        /* process the data that is still left */
+                        process_rx_data(user, pMrk1, pMrk2-pMrk1);
+                    }
+
+                    /* move the binary data after this IAC */
+                    if (pRx < pEnd)
+                    {
+                        memmove(user->net.rxbuf, pRx, pEnd-pRx);
+                        user->net.rxp = user->net.rxbuf + (pEnd-pRx);
+
+                        /* re-call ourselves */
+                        process_rx_buffer(user);
+                    }
+                    else
+                    {
+                        user->net.rxp = user->net.rxbuf;
+                    }
+                    return;     /* NOTE: return */
+
+                case IAC_OK:
+                    /* restore the state */
+                    user->net.rx_proc_state = RX_PROC_STATE_DATA;
+                    /* NOTE: FALL-THROUGH */
+
+                default:
+                    /* just advance the buffer pointer */
+                    pRx += iac_length;
+                    break;
+            }
+
+            if (pMrk1 != pMrk2)
+            {
+                /* process the data that is still left */
+                process_rx_data(user, pMrk1, pMrk2-pMrk1);
+            }
+
+            /* reset the markers */
+            pMrk1 = pMrk2 = pRx;
         }
     }
 
-    /* only if IAC did not reach end of buffer */
-    if (retval != IAC_BUFFER_END)
+    /* reset reception buffer pointers */
+    if (MCCP_USER(user))
     {
-        /* reset buffer pointers */
-        if (MCCP_USER(user))
-        {
-            user->net.mccp_outp = user->net.mccp_out;
-        }
-        else
-        {
-            user->net.rxp = user->net.rxbuf;
-        }
+        user->net.mccp_outp = user->net.mccp_out;
+    }
+    else
+    {
+        user->net.rxp = user->net.rxbuf;
+    }
 
-        if (pRx != pMrk)
-        {
-            /* process the data that is still left */
-            process_rx_data(user, pMrk, pRx-pMrk);
-        }
+    if (pMrk1 != pMrk2)
+    {
+        /* process the data that is still left */
+        process_rx_data(user, pMrk1, pMrk2-pMrk1);
     }
 
     return;
